'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _ernUtil = require('ern-util');

var _ernCore = require('ern-core');

var _utils = require('./utils.js');

var _lodash = require('lodash');

var _lodash2 = _interopRequireDefault(_lodash);

var _shelljs = require('shelljs');

var _shelljs2 = _interopRequireDefault(_shelljs);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { return Promise.resolve(value).then(function (value) { step("next", value); }, function (err) { step("throw", err); }); } } return step("next"); }); }; }

var mustacheView = {};

var npmScopeModuleRe = /(@.*)\/(.*)/;

// =============================================================================
// ern-container-gen entry point
// =============================================================================

// generator: The generator to use along with its config
//  ex :
//  {
//    platform: "android",
//    name: "maven",
//    containerVersion: "1.2.3",
//    mavenRepositoryUrl = ...
//  }
// plugins: Array containing all plugins to be included in the container
//  ex :
//  [{ name: "react-native", version: "0.40.0"}, {name: "react-native-code-push"}]
// miniApps: Array of mini apps to be included in the container
//  ex :
//  [
//    {
//      name: "RnDemoApp",
//      version: "1.0.0"
//    }
//  ]

exports.default = function () {
  var _ref = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee() {
    var _ref2 = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {},
        containerVersion = _ref2.containerVersion,
        nativeAppName = _ref2.nativeAppName,
        platformPath = _ref2.platformPath,
        generator = _ref2.generator,
        plugins = _ref2.plugins,
        miniapps = _ref2.miniapps,
        workingFolder = _ref2.workingFolder,
        pathToYarnLock = _ref2.pathToYarnLock;

    var PLUGINS_DOWNLOAD_FOLDER, OUT_FOLDER, COMPOSITE_MINIAPP_FOLDER, paths, reactNativePlugin, miniApps;
    return regeneratorRuntime.wrap(function _callee$(_context) {
      while (1) {
        switch (_context.prev = _context.next) {
          case 0:
            if (generator.generateContainer) {
              _context.next = 2;
              break;
            }

            throw new Error('The generator miss a generateContainer function !');

          case 2:

            // Folder from which we download all plugins sources (from npm or git)
            PLUGINS_DOWNLOAD_FOLDER = `${workingFolder}/plugins`;
            // Folder where the resulting container project is stored in

            OUT_FOLDER = `${workingFolder}/out`;
            // Folder from which we assemble the miniapps together / run the bundling

            COMPOSITE_MINIAPP_FOLDER = `${workingFolder}/compositeMiniApp`;

            // Contains all interesting folders paths

            paths = {
              // Where the container project hull is stored
              containerHull: `${platformPath}/ern-container-gen/hull`,
              // Where the templates to be used during container generation are stored
              containerTemplates: `${platformPath}/ern-container-gen/templates`,
              // Where we assemble the miniapps together
              compositeMiniApp: COMPOSITE_MINIAPP_FOLDER,
              // Where we download plugins
              pluginsDownloadFolder: PLUGINS_DOWNLOAD_FOLDER,
              // Where we output final generated container
              outFolder: OUT_FOLDER

              // Clean up to start fresh
            };
            _shelljs2.default.rm('-rf', PLUGINS_DOWNLOAD_FOLDER);
            _shelljs2.default.rm('-rf', OUT_FOLDER);
            _shelljs2.default.rm('-rf', COMPOSITE_MINIAPP_FOLDER);
            _shelljs2.default.mkdir('-p', PLUGINS_DOWNLOAD_FOLDER);
            _shelljs2.default.mkdir('-p', `${OUT_FOLDER}/android`);
            _shelljs2.default.mkdir('-p', `${OUT_FOLDER}/ios`);
            (0, _utils.throwIfShellCommandFailed)();

            // Sort the plugin to have consistent ElectrodeContainer.java generated code
            sortPlugins(plugins);

            // Let's make sure that react-native is included (otherwise there is
            // something pretty much wrong)
            reactNativePlugin = _lodash2.default.find(plugins, function (p) {
              return p.name === 'react-native';
            });

            if (reactNativePlugin) {
              _context.next = 17;
              break;
            }

            throw new Error('react-native was not found in plugins list !');

          case 17:
            miniApps = _lodash2.default.map(miniapps, function (miniapp) {
              return {
                name: miniapp.name,
                scope: miniapp.scope,
                version: miniapp.version,
                unscopedName: getUnscopedModuleName(miniapp.name).replace(/-/g, ''),
                pascalCaseName: (0, _utils.capitalizeFirstLetter)(getUnscopedModuleName(miniapp.name)).replace(/-/g, ''),
                localPath: miniapp.path,
                packagePath: miniapp.packageDescriptor
              };
            });


            mustacheView = {
              reactNativeVersion: reactNativePlugin.version,
              nativeAppName,
              miniApps,
              containerVersion
            };
            _context.next = 21;
            return generator.generateContainer(containerVersion, nativeAppName, plugins, miniApps, paths, mustacheView, { pathToYarnLock });

          case 21:
            return _context.abrupt('return', paths);

          case 22:
          case 'end':
            return _context.stop();
        }
      }
    }, _callee, this);
  }));

  function generateContainer() {
    return _ref.apply(this, arguments);
  }

  return generateContainer;
}();

function getUnscopedModuleName(moduleName) {
  return npmScopeModuleRe.test(moduleName) ? npmScopeModuleRe.exec(`${moduleName}`)[2] : moduleName;
}

function sortPlugins(plugins) {
  return plugins.sort(function (a, b) {
    if (a.name < b.name) {
      return -1;
    }
    if (a.name > b.name) {
      return 1;
    }
    return 0;
  });
}
//# sourceMappingURL=generateContainer.js.map