'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _ernCore = require('ern-core');

var _ernUtil = require('ern-util');

var _utils = require('../../utils.js');

var _lodash = require('lodash');

var _lodash2 = _interopRequireDefault(_lodash);

var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _fsReaddirRecursive = require('fs-readdir-recursive');

var _fsReaddirRecursive2 = _interopRequireDefault(_fsReaddirRecursive);

var _shelljs = require('shelljs');

var _shelljs2 = _interopRequireDefault(_shelljs);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { return Promise.resolve(value).then(function (value) { step("next", value); }, function (err) { step("throw", err); }); } } return step("next"); }); }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var ROOT_DIR = _shelljs2.default.pwd();
var DEFAULT_NAMESPACE = 'com.walmartlabs.ern';

var AndroidGenerator = function () {
  function AndroidGenerator() {
    var _ref = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {},
        containerGeneratorConfig = _ref.containerGeneratorConfig,
        _ref$namespace = _ref.namespace,
        namespace = _ref$namespace === undefined ? DEFAULT_NAMESPACE : _ref$namespace;

    _classCallCheck(this, AndroidGenerator);

    this._containerGeneratorConfig = containerGeneratorConfig;
    this._namespace = namespace;
  }

  _createClass(AndroidGenerator, [{
    key: 'generateContainer',
    value: function () {
      var _ref2 = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee(containerVersion, nativeAppName, plugins, miniapps, paths, mustacheView) {
        var _ref3 = arguments.length > 6 && arguments[6] !== undefined ? arguments[6] : {},
            pathToYarnLock = _ref3.pathToYarnLock;

        var mavenPublisher, gitHubPublisher, repoUrl;
        return regeneratorRuntime.wrap(function _callee$(_context) {
          while (1) {
            switch (_context.prev = _context.next) {
              case 0:
                _shelljs2.default.cd(paths.outFolder);
                (0, _utils.throwIfShellCommandFailed)();

                mavenPublisher = this._containerGeneratorConfig.firstAvailableMavenPublisher;

                if (this._containerGeneratorConfig.shouldPublish() && mavenPublisher) {
                  log.debug(`Container will be published to ${mavenPublisher.url}`);
                  if (_ernCore.MavenUtils.isLocalMavenRepo(mavenPublisher.url)) {
                    _ernCore.MavenUtils.createLocalMavenDirectoryIfDoesNotExist();
                  }
                } else {
                  log.warn('Something does not look right, android should always have a default maven publisher.');
                  _ernUtil.Utils.logErrorAndExitProcess(`Something does not look right, android should always have a default maven publisher.`);
                }

                gitHubPublisher = this._containerGeneratorConfig.firstAvailableGitHubPublisher;

                if (!gitHubPublisher) {
                  _context.next = 20;
                  break;
                }

                _context.prev = 6;

                log.debug(`GitHub publisher found. Lets clone the repo before generating the container.`);
                repoUrl = gitHubPublisher.url;

                log.debug(`\n === Generated container will also be published to github
          targetRepoUrl: ${repoUrl}
          containerVersion: ${containerVersion}`);

                log.debug(`First lets clone the repo so we can update it with the newly generated container. Location: ${paths.outFolder}`);
                _context.next = 13;
                return _ernCore.GitUtils.gitClone(repoUrl, { destFolder: 'android' });

              case 13:

                _shelljs2.default.rm('-rf', `${paths.outFolder}/android/*`);
                (0, _utils.throwIfShellCommandFailed)();
                _context.next = 20;
                break;

              case 17:
                _context.prev = 17;
                _context.t0 = _context['catch'](6);

                _ernUtil.Utils.logErrorAndExitProcess('Container generation Failed while cloning the repo. \n Check to see if the entered URL is correct');

              case 20:

                // Enhance mustache view with android specifics
                mustacheView.android = {
                  repository: mavenPublisher ? _ernCore.MavenUtils.targetRepositoryGradleStatement(mavenPublisher.url) : undefined,
                  namespace: this.namespace,
                  miniapps: miniapps

                  //
                  // Go through all ern-container-gen steps

                  // Copy the container hull to output folder and patch it
                  // - Retrieves (download) each plugin from npm or git and inject
                  //   plugin source in container
                  // - Inject configuration code for plugins that expose configuration
                  // - Create activities for MiniApps
                  // - Patch build.gradle for versioning of the container project and
                  //   to specify publication repository target
                };_context.next = 23;
                return this.fillContainerHull(plugins, miniapps, paths, mustacheView);

              case 23:
                _context.next = 25;
                return (0, _utils.bundleMiniApps)(miniapps, paths, 'android', { pathToYarnLock });

              case 25:

                // Rnpm handling
                if (!this._containerGeneratorConfig.ignoreRnpmAssets) {
                  this.copyRnpmAssets(miniapps, paths);
                }

                // Finally, container hull project is fully generated, now let's just
                // build it and publish resulting AAR

                if (!mavenPublisher) {
                  _context.next = 31;
                  break;
                }

                _context.next = 29;
                return mavenPublisher.publish({ workingDir: `${paths.outFolder}/android`, moduleName: `lib` });

              case 29:
                log.debug(`Published com.walmartlabs.ern:${nativeAppName}-ern-container:${containerVersion}`);
                log.debug(`To ${mavenPublisher.url}`);

              case 31:
                if (!gitHubPublisher) {
                  _context.next = 37;
                  break;
                }

                _shelljs2.default.cd(`${paths.outFolder}/android`);
                (0, _utils.throwIfShellCommandFailed)();
                _context.next = 36;
                return gitHubPublisher.publish({ commitMessage: `Container v${containerVersion}`, tag: `v${containerVersion}` });

              case 36:
                log.debug(`Code pushed to ${gitHubPublisher.url}`);

              case 37:
              case 'end':
                return _context.stop();
            }
          }
        }, _callee, this, [[6, 17]]);
      }));

      function generateContainer(_x2, _x3, _x4, _x5, _x6, _x7) {
        return _ref2.apply(this, arguments);
      }

      return generateContainer;
    }()
  }, {
    key: 'fillContainerHull',
    value: function () {
      var _ref4 = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee2(plugins, miniApps, paths, mustacheView) {
        var outputFolder, _iteratorNormalCompletion, _didIteratorError, _iteratorError, _iterator, _step, plugin, pluginConfig, pluginSourcePath, _iteratorNormalCompletion4, _didIteratorError4, _iteratorError4, _iterator4, _step4, dependency, files, _iteratorNormalCompletion2, _didIteratorError2, _iteratorError2, _iterator2, _step2, file, _iteratorNormalCompletion3, _didIteratorError3, _iteratorError3, _iterator3, _step3, miniApp, tmpMiniAppView, activityFileName;

        return regeneratorRuntime.wrap(function _callee2$(_context2) {
          while (1) {
            switch (_context2.prev = _context2.next) {
              case 0:
                _context2.prev = 0;

                log.debug(`[=== Starting container hull filling ===]`);

                log.debug(`> cd ${ROOT_DIR}`);
                _shelljs2.default.cd(`${ROOT_DIR}`);
                (0, _utils.throwIfShellCommandFailed)();

                outputFolder = `${paths.outFolder}/android`;


                log.debug(`> cp -R ${paths.containerHull}/android/* ${outputFolder}`);
                _shelljs2.default.cp('-R', `${paths.containerHull}/android/*`, outputFolder);
                _shelljs2.default.cp('-R', `${paths.containerHull}/android/.*`, outputFolder);
                (0, _utils.throwIfShellCommandFailed)();

                _context2.next = 12;
                return this.buildAndroidPluginsViews(plugins, mustacheView);

              case 12:
                _context2.next = 14;
                return this.addAndroidPluginHookClasses(plugins, paths);

              case 14:
                _iteratorNormalCompletion = true;
                _didIteratorError = false;
                _iteratorError = undefined;
                _context2.prev = 17;
                _iterator = plugins[Symbol.iterator]();

              case 19:
                if (_iteratorNormalCompletion = (_step = _iterator.next()).done) {
                  _context2.next = 67;
                  break;
                }

                plugin = _step.value;
                _context2.next = 23;
                return _ernCore.manifest.getPluginConfig(plugin);

              case 23:
                pluginConfig = _context2.sent;
                pluginSourcePath = void 0;

                if (!(plugin.name === 'react-native')) {
                  _context2.next = 27;
                  break;
                }

                return _context2.abrupt('continue', 64);

              case 27:
                if (pluginConfig.android) {
                  _context2.next = 30;
                  break;
                }

                log.warn(`Skipping ${plugin.name} as it does not have an Android configuration`);
                return _context2.abrupt('continue', 64);

              case 30:
                log.debug(`> cd ${paths.pluginsDownloadFolder}`);
                _shelljs2.default.cd(`${paths.pluginsDownloadFolder}`);
                (0, _utils.throwIfShellCommandFailed)();
                _context2.next = 35;
                return (0, _utils.downloadPluginSource)(pluginConfig.origin);

              case 35:
                pluginSourcePath = _context2.sent;

                if (pluginSourcePath) {
                  _context2.next = 38;
                  break;
                }

                throw new Error(`Was not able to download ${plugin.name}`);

              case 38:
                log.debug(`> cd ${pluginSourcePath}/${pluginConfig.android.root}`);
                _shelljs2.default.cd(`${pluginSourcePath}/${pluginConfig.android.root}`);
                (0, _utils.throwIfShellCommandFailed)();
                if (pluginConfig.android.moduleName) {
                  log.debug(`> cp -R ${pluginConfig.android.moduleName}/src/main/java ${outputFolder}/lib/src/main`);
                  _shelljs2.default.cp('-R', `${pluginConfig.android.moduleName}/src/main/java`, `${outputFolder}/lib/src/main`);
                  (0, _utils.throwIfShellCommandFailed)();
                } else {
                  log.debug(`> cp -R src/main/java ${outputFolder}/lib/src/main`);
                  _shelljs2.default.cp('-R', `src/main/java`, `${outputFolder}/lib/src/main`);
                  (0, _utils.throwIfShellCommandFailed)();
                }

                if (!pluginConfig.android) {
                  _context2.next = 64;
                  break;
                }

                if (pluginConfig.android.copy) {
                  (0, _ernCore.handleCopyDirective)(pluginSourcePath, outputFolder, pluginConfig.android.copy);
                }

                if (!pluginConfig.android.dependencies) {
                  _context2.next = 64;
                  break;
                }

                _iteratorNormalCompletion4 = true;
                _didIteratorError4 = false;
                _iteratorError4 = undefined;
                _context2.prev = 48;

                for (_iterator4 = pluginConfig.android.dependencies[Symbol.iterator](); !(_iteratorNormalCompletion4 = (_step4 = _iterator4.next()).done); _iteratorNormalCompletion4 = true) {
                  dependency = _step4.value;

                  log.debug(`Adding compile '${dependency}'`);
                  mustacheView.pluginCompile.push({
                    'compileStatement': `compile '${dependency}'`
                  });
                }
                _context2.next = 56;
                break;

              case 52:
                _context2.prev = 52;
                _context2.t0 = _context2['catch'](48);
                _didIteratorError4 = true;
                _iteratorError4 = _context2.t0;

              case 56:
                _context2.prev = 56;
                _context2.prev = 57;

                if (!_iteratorNormalCompletion4 && _iterator4.return) {
                  _iterator4.return();
                }

              case 59:
                _context2.prev = 59;

                if (!_didIteratorError4) {
                  _context2.next = 62;
                  break;
                }

                throw _iteratorError4;

              case 62:
                return _context2.finish(59);

              case 63:
                return _context2.finish(56);

              case 64:
                _iteratorNormalCompletion = true;
                _context2.next = 19;
                break;

              case 67:
                _context2.next = 73;
                break;

              case 69:
                _context2.prev = 69;
                _context2.t1 = _context2['catch'](17);
                _didIteratorError = true;
                _iteratorError = _context2.t1;

              case 73:
                _context2.prev = 73;
                _context2.prev = 74;

                if (!_iteratorNormalCompletion && _iterator.return) {
                  _iterator.return();
                }

              case 76:
                _context2.prev = 76;

                if (!_didIteratorError) {
                  _context2.next = 79;
                  break;
                }

                throw _iteratorError;

              case 79:
                return _context2.finish(76);

              case 80:
                return _context2.finish(73);

              case 81:

                log.debug(`Patching hull`);
                files = (0, _fsReaddirRecursive2.default)(`${outputFolder}`, function (f) {
                  return !f.endsWith('.jar') && !f.endsWith('.aar') && !f.endsWith('.git');
                });
                _iteratorNormalCompletion2 = true;
                _didIteratorError2 = false;
                _iteratorError2 = undefined;
                _context2.prev = 86;
                _iterator2 = files[Symbol.iterator]();

              case 88:
                if (_iteratorNormalCompletion2 = (_step2 = _iterator2.next()).done) {
                  _context2.next = 99;
                  break;
                }

                file = _step2.value;

                if (!(file.startsWith(`lib/src/main/java/com`) && !file.startsWith(`lib/src/main/java/com/walmartlabs/ern/container`))) {
                  _context2.next = 93;
                  break;
                }

                // We don't want to Mustache process library files. It can lead to bad things
                // We just want to process container specific code (which contains mustache templates)
                log.debug(`Skipping mustaching of ${file}`);
                return _context2.abrupt('continue', 96);

              case 93:
                log.debug(`Mustaching ${file}`);
                _context2.next = 96;
                return _ernUtil.mustacheUtils.mustacheRenderToOutputFileUsingTemplateFile(`${outputFolder}/${file}`, mustacheView, `${outputFolder}/${file}`);

              case 96:
                _iteratorNormalCompletion2 = true;
                _context2.next = 88;
                break;

              case 99:
                _context2.next = 105;
                break;

              case 101:
                _context2.prev = 101;
                _context2.t2 = _context2['catch'](86);
                _didIteratorError2 = true;
                _iteratorError2 = _context2.t2;

              case 105:
                _context2.prev = 105;
                _context2.prev = 106;

                if (!_iteratorNormalCompletion2 && _iterator2.return) {
                  _iterator2.return();
                }

              case 108:
                _context2.prev = 108;

                if (!_didIteratorError2) {
                  _context2.next = 111;
                  break;
                }

                throw _iteratorError2;

              case 111:
                return _context2.finish(108);

              case 112:
                return _context2.finish(105);

              case 113:

                // Create mini app activities
                log.debug(`Creating miniapp activities`);
                _iteratorNormalCompletion3 = true;
                _didIteratorError3 = false;
                _iteratorError3 = undefined;
                _context2.prev = 117;
                _iterator3 = miniApps[Symbol.iterator]();

              case 119:
                if (_iteratorNormalCompletion3 = (_step3 = _iterator3.next()).done) {
                  _context2.next = 129;
                  break;
                }

                miniApp = _step3.value;
                tmpMiniAppView = {
                  miniAppName: miniApp.unscopedName,
                  pascalCaseMiniAppName: miniApp.pascalCaseName
                };
                activityFileName = `${tmpMiniAppView.pascalCaseMiniAppName}Activity.java`;


                log.debug(`Creating ${activityFileName}`);
                _context2.next = 126;
                return _ernUtil.mustacheUtils.mustacheRenderToOutputFileUsingTemplateFile(`${paths.containerTemplates}/android/MiniAppActivity.mustache`, tmpMiniAppView, `${outputFolder}/lib/src/main/java/com/walmartlabs/ern/container/miniapps/${activityFileName}`);

              case 126:
                _iteratorNormalCompletion3 = true;
                _context2.next = 119;
                break;

              case 129:
                _context2.next = 135;
                break;

              case 131:
                _context2.prev = 131;
                _context2.t3 = _context2['catch'](117);
                _didIteratorError3 = true;
                _iteratorError3 = _context2.t3;

              case 135:
                _context2.prev = 135;
                _context2.prev = 136;

                if (!_iteratorNormalCompletion3 && _iterator3.return) {
                  _iterator3.return();
                }

              case 138:
                _context2.prev = 138;

                if (!_didIteratorError3) {
                  _context2.next = 141;
                  break;
                }

                throw _iteratorError3;

              case 141:
                return _context2.finish(138);

              case 142:
                return _context2.finish(135);

              case 143:

                log.debug(`[=== Completed container hull filling ===]`);
                _context2.next = 150;
                break;

              case 146:
                _context2.prev = 146;
                _context2.t4 = _context2['catch'](0);

                log.error('[fillContainerHull] Something went wrong: ' + _context2.t4);
                throw _context2.t4;

              case 150:
              case 'end':
                return _context2.stop();
            }
          }
        }, _callee2, this, [[0, 146], [17, 69, 73, 81], [48, 52, 56, 64], [57,, 59, 63], [74,, 76, 80], [86, 101, 105, 113], [106,, 108, 112], [117, 131, 135, 143], [136,, 138, 142]]);
      }));

      function fillContainerHull(_x9, _x10, _x11, _x12) {
        return _ref4.apply(this, arguments);
      }

      return fillContainerHull;
    }()
  }, {
    key: 'copyRnpmAssets',
    value: function copyRnpmAssets(miniApps, paths) {
      var outputFolder = _path2.default.join(paths.outFolder, 'android');
      // Case of local container for runner
      if (miniApps.length === 1 && miniApps[0].localPath) {
        this.copyRnpmAssetsFromMiniAppPath(miniApps[0].localPath, outputFolder);
      } else {
        var _iteratorNormalCompletion5 = true;
        var _didIteratorError5 = false;
        var _iteratorError5 = undefined;

        try {
          for (var _iterator5 = miniApps[Symbol.iterator](), _step5; !(_iteratorNormalCompletion5 = (_step5 = _iterator5.next()).done); _iteratorNormalCompletion5 = true) {
            var miniApp = _step5.value;

            var miniAppPath = _path2.default.join(paths.compositeMiniApp, 'node_modules', miniApp.scope ? `@${miniApp.scope}` : '', miniApp.name);
            this.copyRnpmAssetsFromMiniAppPath(miniAppPath, outputFolder);
          }
        } catch (err) {
          _didIteratorError5 = true;
          _iteratorError5 = err;
        } finally {
          try {
            if (!_iteratorNormalCompletion5 && _iterator5.return) {
              _iterator5.return();
            }
          } finally {
            if (_didIteratorError5) {
              throw _iteratorError5;
            }
          }
        }
      }
    }
  }, {
    key: 'copyRnpmAssetsFromMiniAppPath',
    value: function copyRnpmAssetsFromMiniAppPath(miniAppPath, outputPath) {
      var packageJson = JSON.parse(_fs2.default.readFileSync(_path2.default.join(miniAppPath, 'package.json'), 'utf-8'));
      if (packageJson.rnpm && packageJson.rnpm.assets) {
        var _iteratorNormalCompletion6 = true;
        var _didIteratorError6 = false;
        var _iteratorError6 = undefined;

        try {
          for (var _iterator6 = packageJson.rnpm.assets[Symbol.iterator](), _step6; !(_iteratorNormalCompletion6 = (_step6 = _iterator6.next()).done); _iteratorNormalCompletion6 = true) {
            var assetDirectoryName = _step6.value;

            (0, _ernCore.handleCopyDirective)(miniAppPath, outputPath, [{ source: `${assetDirectoryName}/*`, dest: `lib/src/main/assets/${assetDirectoryName.toLowerCase()}` }]);
          }
        } catch (err) {
          _didIteratorError6 = true;
          _iteratorError6 = err;
        } finally {
          try {
            if (!_iteratorNormalCompletion6 && _iterator6.return) {
              _iterator6.return();
            }
          } finally {
            if (_didIteratorError6) {
              throw _iteratorError6;
            }
          }
        }
      }
    }
  }, {
    key: 'addAndroidPluginHookClasses',
    value: function () {
      var _ref5 = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee3(plugins, paths) {
        var _iteratorNormalCompletion7, _didIteratorError7, _iteratorError7, _iterator7, _step7, plugin, pluginConfig, androidPluginHook;

        return regeneratorRuntime.wrap(function _callee3$(_context3) {
          while (1) {
            switch (_context3.prev = _context3.next) {
              case 0:
                _context3.prev = 0;

                log.debug(`[=== Adding plugin hook classes ===]`);

                _iteratorNormalCompletion7 = true;
                _didIteratorError7 = false;
                _iteratorError7 = undefined;
                _context3.prev = 5;
                _iterator7 = plugins[Symbol.iterator]();

              case 7:
                if (_iteratorNormalCompletion7 = (_step7 = _iterator7.next()).done) {
                  _context3.next = 28;
                  break;
                }

                plugin = _step7.value;

                if (!(plugin.name === 'react-native')) {
                  _context3.next = 11;
                  break;
                }

                return _context3.abrupt('continue', 25);

              case 11:
                log.debug(`Handling ${plugin.name}`);
                _context3.next = 14;
                return _ernCore.manifest.getPluginConfig(plugin);

              case 14:
                pluginConfig = _context3.sent;

                if (pluginConfig.android) {
                  _context3.next = 18;
                  break;
                }

                log.warn(`Skipping ${plugin.name} as it does not have an Android configuration`);
                return _context3.abrupt('continue', 25);

              case 18:
                androidPluginHook = pluginConfig.android.pluginHook;

                if (!androidPluginHook) {
                  _context3.next = 25;
                  break;
                }

                log.debug(`Adding ${androidPluginHook.name}.java`);

                if (pluginConfig.path) {
                  _context3.next = 23;
                  break;
                }

                throw new Error(`No plugin config path was set. Cannot proceed.`);

              case 23:
                _shelljs2.default.cp(`${pluginConfig.path}/${androidPluginHook.name}.java`, `${paths.outFolder}/android/lib/src/main/java/com/walmartlabs/ern/container/plugins/`);
                (0, _utils.throwIfShellCommandFailed)();

              case 25:
                _iteratorNormalCompletion7 = true;
                _context3.next = 7;
                break;

              case 28:
                _context3.next = 34;
                break;

              case 30:
                _context3.prev = 30;
                _context3.t0 = _context3['catch'](5);
                _didIteratorError7 = true;
                _iteratorError7 = _context3.t0;

              case 34:
                _context3.prev = 34;
                _context3.prev = 35;

                if (!_iteratorNormalCompletion7 && _iterator7.return) {
                  _iterator7.return();
                }

              case 37:
                _context3.prev = 37;

                if (!_didIteratorError7) {
                  _context3.next = 40;
                  break;
                }

                throw _iteratorError7;

              case 40:
                return _context3.finish(37);

              case 41:
                return _context3.finish(34);

              case 42:

                log.debug(`[=== Done adding plugin hook classes ===]`);
                _context3.next = 49;
                break;

              case 45:
                _context3.prev = 45;
                _context3.t1 = _context3['catch'](0);

                log.error('[addAndroidPluginHookClasses] Something went wrong: ' + _context3.t1);
                throw _context3.t1;

              case 49:
              case 'end':
                return _context3.stop();
            }
          }
        }, _callee3, this, [[0, 45], [5, 30, 34, 42], [35,, 37, 41]]);
      }));

      function addAndroidPluginHookClasses(_x13, _x14) {
        return _ref5.apply(this, arguments);
      }

      return addAndroidPluginHookClasses;
    }()
  }, {
    key: 'buildAndroidPluginsViews',
    value: function () {
      var _ref6 = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee4(plugins, mustacheView) {
        var pluginsView, _iteratorNormalCompletion8, _didIteratorError8, _iteratorError8, _iterator8, _step8, plugin, pluginConfig, androidPluginHook, reactNativePlugin;

        return regeneratorRuntime.wrap(function _callee4$(_context4) {
          while (1) {
            switch (_context4.prev = _context4.next) {
              case 0:
                _context4.prev = 0;
                pluginsView = [];
                _iteratorNormalCompletion8 = true;
                _didIteratorError8 = false;
                _iteratorError8 = undefined;
                _context4.prev = 5;
                _iterator8 = plugins[Symbol.iterator]();

              case 7:
                if (_iteratorNormalCompletion8 = (_step8 = _iterator8.next()).done) {
                  _context4.next = 22;
                  break;
                }

                plugin = _step8.value;

                if (!(plugin.name === 'react-native')) {
                  _context4.next = 11;
                  break;
                }

                return _context4.abrupt('continue', 19);

              case 11:
                _context4.next = 13;
                return _ernCore.manifest.getPluginConfig(plugin);

              case 13:
                pluginConfig = _context4.sent;

                if (pluginConfig.android) {
                  _context4.next = 17;
                  break;
                }

                log.warn(`${plugin.name} does not have any injection configuration for Android`);
                return _context4.abrupt('continue', 19);

              case 17:
                androidPluginHook = pluginConfig.android.pluginHook;

                if (androidPluginHook) {
                  log.debug(`Hooking ${plugin.scopedName} plugin`);
                  pluginsView.push({
                    'name': androidPluginHook.name,
                    'lcname': androidPluginHook.name.charAt(0).toLowerCase() + androidPluginHook.name.slice(1),
                    'configurable': androidPluginHook.configurable
                  });
                }

              case 19:
                _iteratorNormalCompletion8 = true;
                _context4.next = 7;
                break;

              case 22:
                _context4.next = 28;
                break;

              case 24:
                _context4.prev = 24;
                _context4.t0 = _context4['catch'](5);
                _didIteratorError8 = true;
                _iteratorError8 = _context4.t0;

              case 28:
                _context4.prev = 28;
                _context4.prev = 29;

                if (!_iteratorNormalCompletion8 && _iterator8.return) {
                  _iterator8.return();
                }

              case 31:
                _context4.prev = 31;

                if (!_didIteratorError8) {
                  _context4.next = 34;
                  break;
                }

                throw _iteratorError8;

              case 34:
                return _context4.finish(31);

              case 35:
                return _context4.finish(28);

              case 36:

                mustacheView.plugins = pluginsView;

                mustacheView.pluginCompile = [];
                reactNativePlugin = _lodash2.default.find(plugins, function (p) {
                  return p.name === 'react-native';
                });

                if (reactNativePlugin) {
                  log.debug(`Will inject: compile 'com.walmartlabs.ern:react-native:${reactNativePlugin.version}'`);
                  mustacheView.pluginCompile.push({
                    'compileStatement': `compile 'com.walmartlabs.ern:react-native:${reactNativePlugin.version}'`
                  });
                }
                _context4.next = 46;
                break;

              case 42:
                _context4.prev = 42;
                _context4.t1 = _context4['catch'](0);

                log.error('[buildAndroidPluginsViews] Something went wrong: ' + _context4.t1);
                throw _context4.t1;

              case 46:
              case 'end':
                return _context4.stop();
            }
          }
        }, _callee4, this, [[0, 42], [5, 24, 28, 36], [29,, 31, 35]]);
      }));

      function buildAndroidPluginsViews(_x15, _x16) {
        return _ref6.apply(this, arguments);
      }

      return buildAndroidPluginsViews;
    }()
  }, {
    key: 'name',
    get: function get() {
      return 'AndroidGenerator';
    }
  }, {
    key: 'platform',
    get: function get() {
      return 'android';
    }
  }, {
    key: 'namespace',
    get: function get() {
      return this._namespace;
    }
  }]);

  return AndroidGenerator;
}();

exports.default = AndroidGenerator;
//# sourceMappingURL=AndroidGenerator.js.map