'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _ernCore = require('ern-core');

var _ernUtil = require('ern-util');

var _utils = require('../../utils.js');

var _lodash = require('lodash');

var _lodash2 = _interopRequireDefault(_lodash);

var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

var _path2 = require('path');

var _path3 = _interopRequireDefault(_path2);

var _shelljs = require('shelljs');

var _shelljs2 = _interopRequireDefault(_shelljs);

var _xcodeErn = require('xcode-ern');

var _xcodeErn2 = _interopRequireDefault(_xcodeErn);

var _fsReaddirRecursive = require('fs-readdir-recursive');

var _fsReaddirRecursive2 = _interopRequireDefault(_fsReaddirRecursive);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { return Promise.resolve(value).then(function (value) { step("next", value); }, function (err) { step("throw", err); }); } } return step("next"); }); }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var ROOT_DIR = _shelljs2.default.pwd();

var IosGenerator = function () {
  function IosGenerator(containerGeneratorConfig) {
    _classCallCheck(this, IosGenerator);

    this._containerGeneratorConfig = containerGeneratorConfig;
  }

  _createClass(IosGenerator, [{
    key: 'generateContainer',
    value: function () {
      var _ref = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee(containerVersion, nativeAppName, plugins, miniapps, paths, mustacheView) {
        var _ref2 = arguments.length > 6 && arguments[6] !== undefined ? arguments[6] : {},
            pathToYarnLock = _ref2.pathToYarnLock;

        var gitHubPublisher, repoUrl;
        return regeneratorRuntime.wrap(function _callee$(_context) {
          while (1) {
            switch (_context.prev = _context.next) {
              case 0:
                _context.prev = 0;

                _shelljs2.default.cd(paths.outFolder);
                (0, _utils.throwIfShellCommandFailed)();

                gitHubPublisher = void 0;

                if (!this._containerGeneratorConfig.shouldPublish()) {
                  _context.next = 18;
                  break;
                }

                if (!(gitHubPublisher = this._containerGeneratorConfig.firstAvailableGitHubPublisher)) {
                  _context.next = 15;
                  break;
                }

                repoUrl = gitHubPublisher.url;

                log.debug(`\n === Using github publisher
          targetRepoUrl: ${repoUrl}
          containerVersion: ${containerVersion}`);

                log.debug(`First lets clone the repo so we can update it with the newly generated container. Location: ${paths.outFolder}`);
                _context.next = 11;
                return _ernCore.GitUtils.gitClone(repoUrl, { destFolder: 'ios' });

              case 11:

                _shelljs2.default.rm('-rf', `${paths.outFolder}/ios/*`);
                (0, _utils.throwIfShellCommandFailed)();
                _context.next = 16;
                break;

              case 15:
                log.warn('Looks like we are missing a GitHub publisher. Currently only GitHub publisher is supported.');

              case 16:
                _context.next = 19;
                break;

              case 18:
                log.debug('Will not publish since there is no publisher provided to generator.');

              case 19:
                _context.next = 21;
                return this.fillContainerHull(plugins, miniapps, paths, mustacheView);

              case 21:
                if (!(miniapps.length > 0)) {
                  _context.next = 24;
                  break;
                }

                _context.next = 24;
                return (0, _utils.bundleMiniApps)(miniapps, paths, 'ios', { pathToYarnLock });

              case 24:

                // Copy potential rnpm provided assets
                if (!this._containerGeneratorConfig.ignoreRnpmAssets) {
                  this.copyRnpmAssets(miniapps, paths);
                }

                // Publish resulting container to git repo

                if (!gitHubPublisher) {
                  _context.next = 31;
                  break;
                }

                _shelljs2.default.cd(`${paths.outFolder}/ios`);
                (0, _utils.throwIfShellCommandFailed)();
                log.debug(`Publish generated container[v${containerVersion}] to git repo: ${gitHubPublisher.url}`);
                _context.next = 31;
                return gitHubPublisher.publish({ commitMessage: `Container v${containerVersion}`, tag: `v${containerVersion}` });

              case 31:

                log.debug(`Container generation completed!`);
                _context.next = 38;
                break;

              case 34:
                _context.prev = 34;
                _context.t0 = _context['catch'](0);

                log.error(`Something went wrong. Aborting ern-container-gen: ${_context.t0}`);
                console.trace(_context.t0);

              case 38:
              case 'end':
                return _context.stop();
            }
          }
        }, _callee, this, [[0, 34]]);
      }));

      function generateContainer(_x, _x2, _x3, _x4, _x5, _x6) {
        return _ref.apply(this, arguments);
      }

      return generateContainer;
    }()
  }, {
    key: 'copyRnpmAssets',
    value: function copyRnpmAssets(miniApps, paths) {
      var outputFolder = _path3.default.join(paths.outFolder, 'ios');
      // Case of local container for runner
      if (miniApps.length === 1 && miniApps[0].localPath) {
        this.copyRnpmAssetsFromMiniAppPath(miniApps[0].localPath, outputFolder);
      } else {
        var _iteratorNormalCompletion = true;
        var _didIteratorError = false;
        var _iteratorError = undefined;

        try {
          for (var _iterator = miniApps[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
            var miniApp = _step.value;

            var miniAppPath = _path3.default.join(paths.compositeMiniApp, 'node_modules', miniApp.scope ? `@${miniApp.scope}` : '', miniApp.name);
            this.copyRnpmAssetsFromMiniAppPath(miniAppPath, outputFolder);
          }
        } catch (err) {
          _didIteratorError = true;
          _iteratorError = err;
        } finally {
          try {
            if (!_iteratorNormalCompletion && _iterator.return) {
              _iterator.return();
            }
          } finally {
            if (_didIteratorError) {
              throw _iteratorError;
            }
          }
        }
      }
      this.addResources(outputFolder);
    }
  }, {
    key: 'addResources',
    value: function () {
      var _ref3 = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee2(outputFolder) {
        var containerProjectPath, containerIosProject;
        return regeneratorRuntime.wrap(function _callee2$(_context2) {
          while (1) {
            switch (_context2.prev = _context2.next) {
              case 0:
                log.debug(`=== ios: adding resources for miniapps`);
                containerProjectPath = `${outputFolder}/ElectrodeContainer.xcodeproj/project.pbxproj`;
                _context2.next = 4;
                return this.getIosContainerProject(containerProjectPath);

              case 4:
                containerIosProject = _context2.sent;


                (0, _fsReaddirRecursive2.default)(`${outputFolder}/ElectrodeContainer/Resources`, function (resourceFile) {
                  containerIosProject.addResourceFile(`Resources/${resourceFile}`, null, containerIosProject.findPBXGroupKey({ name: 'Resources' }));
                });
                log.debug(`---iOS: Finished adding resource files. `);

                _fs2.default.writeFileSync(containerProjectPath, containerIosProject.writeSync());

              case 8:
              case 'end':
                return _context2.stop();
            }
          }
        }, _callee2, this);
      }));

      function addResources(_x8) {
        return _ref3.apply(this, arguments);
      }

      return addResources;
    }()
  }, {
    key: 'copyRnpmAssetsFromMiniAppPath',
    value: function copyRnpmAssetsFromMiniAppPath(miniAppPath, outputPath) {
      var packageJson = JSON.parse(_fs2.default.readFileSync(_path3.default.join(miniAppPath, 'package.json'), 'utf-8'));
      if (packageJson.rnpm && packageJson.rnpm.assets) {
        var _iteratorNormalCompletion2 = true;
        var _didIteratorError2 = false;
        var _iteratorError2 = undefined;

        try {
          for (var _iterator2 = packageJson.rnpm.assets[Symbol.iterator](), _step2; !(_iteratorNormalCompletion2 = (_step2 = _iterator2.next()).done); _iteratorNormalCompletion2 = true) {
            var assetDirectoryName = _step2.value;

            (0, _ernCore.handleCopyDirective)(miniAppPath, outputPath, [{ source: `${assetDirectoryName}/*`, dest: `ElectrodeContainer/Resources` }]);
          }
        } catch (err) {
          _didIteratorError2 = true;
          _iteratorError2 = err;
        } finally {
          try {
            if (!_iteratorNormalCompletion2 && _iterator2.return) {
              _iterator2.return();
            }
          } finally {
            if (_didIteratorError2) {
              throw _iteratorError2;
            }
          }
        }
      }
    }
  }, {
    key: 'fillContainerHull',
    value: function () {
      var _ref4 = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee3(plugins, miniApps, paths, mustacheView) {
        var _this = this;

        var outputFolder, files, _iteratorNormalCompletion3, _didIteratorError3, _iteratorError3, _iterator3, _step3, file, containerProjectPath, containerLibrariesPath, containerIosProject, electrodeContainerTarget, _iteratorNormalCompletion4, _didIteratorError4, _iteratorError4, _iterator4, _step4, plugin, pluginConfig, pluginSourcePath, _iteratorNormalCompletion5, _didIteratorError5, _iteratorError5, _iterator5, _step5, copy, _iteratorNormalCompletion6, _didIteratorError6, _iteratorError6, _iterator6, _step6, r, fileContent, patchedFileContent, _loop, _iteratorNormalCompletion7, _didIteratorError7, _iteratorError7, _iterator7, _step7, source, _iteratorNormalCompletion9, _didIteratorError9, _iteratorError9, _iterator9, _step9, header, headerPath, _iteratorNormalCompletion10, _didIteratorError10, _iteratorError10, _iterator10, _step10, _file, _iteratorNormalCompletion11, _didIteratorError11, _iteratorError11, _iterator11, _step11, framework, _iteratorNormalCompletion12, _didIteratorError12, _iteratorError12, _iterator12, _step12, project, projectAbsolutePath, _iteratorNormalCompletion13, _didIteratorError13, _iteratorError13, _iterator13, _step13, lib, _iteratorNormalCompletion14, _didIteratorError14, _iteratorError14, _iterator14, _step14, _path;

        return regeneratorRuntime.wrap(function _callee3$(_context3) {
          while (1) {
            switch (_context3.prev = _context3.next) {
              case 0:
                log.debug(`[=== Starting container hull filling ===]`);
                _shelljs2.default.cd(`${ROOT_DIR}`);
                (0, _utils.throwIfShellCommandFailed)();

                outputFolder = `${paths.outFolder}/ios`;


                log.debug(`Creating out folder and copying Container Hull to it`);
                _shelljs2.default.cp('-R', `${paths.containerHull}/ios`, paths.outFolder);
                (0, _utils.throwIfShellCommandFailed)();

                _context3.next = 9;
                return this.buildiOSPluginsViews(plugins, mustacheView);

              case 9:

                log.debug(`---iOS: reading template files to be rendered for plugins`);
                files = (0, _fsReaddirRecursive2.default)(`${outputFolder}`, function (f) {
                  return f;
                });
                _iteratorNormalCompletion3 = true;
                _didIteratorError3 = false;
                _iteratorError3 = undefined;
                _context3.prev = 14;
                _iterator3 = files[Symbol.iterator]();

              case 16:
                if (_iteratorNormalCompletion3 = (_step3 = _iterator3.next()).done) {
                  _context3.next = 24;
                  break;
                }

                file = _step3.value;

                if (!(file.endsWith('.h') || file.endsWith('.m'))) {
                  _context3.next = 21;
                  break;
                }

                _context3.next = 21;
                return _ernUtil.mustacheUtils.mustacheRenderToOutputFileUsingTemplateFile(`${outputFolder}/${file}`, mustacheView, `${outputFolder}/${file}`);

              case 21:
                _iteratorNormalCompletion3 = true;
                _context3.next = 16;
                break;

              case 24:
                _context3.next = 30;
                break;

              case 26:
                _context3.prev = 26;
                _context3.t0 = _context3['catch'](14);
                _didIteratorError3 = true;
                _iteratorError3 = _context3.t0;

              case 30:
                _context3.prev = 30;
                _context3.prev = 31;

                if (!_iteratorNormalCompletion3 && _iterator3.return) {
                  _iterator3.return();
                }

              case 33:
                _context3.prev = 33;

                if (!_didIteratorError3) {
                  _context3.next = 36;
                  break;
                }

                throw _iteratorError3;

              case 36:
                return _context3.finish(33);

              case 37:
                return _context3.finish(30);

              case 38:
                containerProjectPath = `${outputFolder}/ElectrodeContainer.xcodeproj/project.pbxproj`;
                containerLibrariesPath = `${outputFolder}/ElectrodeContainer/Libraries`;
                _context3.next = 42;
                return this.getIosContainerProject(containerProjectPath);

              case 42:
                containerIosProject = _context3.sent;
                electrodeContainerTarget = containerIosProject.findTargetKey('ElectrodeContainer');
                _iteratorNormalCompletion4 = true;
                _didIteratorError4 = false;
                _iteratorError4 = undefined;
                _context3.prev = 47;
                _iterator4 = plugins[Symbol.iterator]();

              case 49:
                if (_iteratorNormalCompletion4 = (_step4 = _iterator4.next()).done) {
                  _context3.next = 249;
                  break;
                }

                plugin = _step4.value;
                _context3.next = 53;
                return _ernCore.manifest.getPluginConfig(plugin);

              case 53:
                pluginConfig = _context3.sent;

                _shelljs2.default.cd(`${paths.pluginsDownloadFolder}`);
                (0, _utils.throwIfShellCommandFailed)();

                if (!pluginConfig.ios) {
                  _context3.next = 246;
                  break;
                }

                log.debug(`Retrieving ${plugin.scopedName}`);
                _context3.next = 60;
                return (0, _utils.downloadPluginSource)(pluginConfig.origin);

              case 60:
                pluginSourcePath = _context3.sent;

                if (pluginSourcePath) {
                  _context3.next = 63;
                  break;
                }

                throw new Error(`Was not able to download ${plugin.scopedName}`);

              case 63:
                if (!pluginConfig.ios.copy) {
                  _context3.next = 84;
                  break;
                }

                _iteratorNormalCompletion5 = true;
                _didIteratorError5 = false;
                _iteratorError5 = undefined;
                _context3.prev = 67;

                for (_iterator5 = pluginConfig.ios.copy[Symbol.iterator](); !(_iteratorNormalCompletion5 = (_step5 = _iterator5.next()).done); _iteratorNormalCompletion5 = true) {
                  copy = _step5.value;

                  if (this.switchToOldDirectoryStructure(pluginSourcePath, copy.source)) {
                    log.debug(`Handling copy directive: Falling back to old directory structure for API(Backward compatibility)`);
                    copy.source = 'IOS/IOS/Classes/SwaggersAPIs/*';
                  }
                }
                _context3.next = 75;
                break;

              case 71:
                _context3.prev = 71;
                _context3.t1 = _context3['catch'](67);
                _didIteratorError5 = true;
                _iteratorError5 = _context3.t1;

              case 75:
                _context3.prev = 75;
                _context3.prev = 76;

                if (!_iteratorNormalCompletion5 && _iterator5.return) {
                  _iterator5.return();
                }

              case 78:
                _context3.prev = 78;

                if (!_didIteratorError5) {
                  _context3.next = 81;
                  break;
                }

                throw _iteratorError5;

              case 81:
                return _context3.finish(78);

              case 82:
                return _context3.finish(75);

              case 83:
                (0, _ernCore.handleCopyDirective)(pluginSourcePath, outputFolder, pluginConfig.ios.copy);

              case 84:
                if (!pluginConfig.ios.replaceInFile) {
                  _context3.next = 104;
                  break;
                }

                _iteratorNormalCompletion6 = true;
                _didIteratorError6 = false;
                _iteratorError6 = undefined;
                _context3.prev = 88;

                for (_iterator6 = pluginConfig.ios.replaceInFile[Symbol.iterator](); !(_iteratorNormalCompletion6 = (_step6 = _iterator6.next()).done); _iteratorNormalCompletion6 = true) {
                  r = _step6.value;
                  fileContent = _fs2.default.readFileSync(`${outputFolder}/${r.path}`, 'utf8');
                  patchedFileContent = fileContent.replace(r.string, r.replaceWith);

                  _fs2.default.writeFileSync(`${outputFolder}/${r.path}`, patchedFileContent, { encoding: 'utf8' });
                }
                _context3.next = 96;
                break;

              case 92:
                _context3.prev = 92;
                _context3.t2 = _context3['catch'](88);
                _didIteratorError6 = true;
                _iteratorError6 = _context3.t2;

              case 96:
                _context3.prev = 96;
                _context3.prev = 97;

                if (!_iteratorNormalCompletion6 && _iterator6.return) {
                  _iterator6.return();
                }

              case 99:
                _context3.prev = 99;

                if (!_didIteratorError6) {
                  _context3.next = 102;
                  break;
                }

                throw _iteratorError6;

              case 102:
                return _context3.finish(99);

              case 103:
                return _context3.finish(96);

              case 104:
                if (!pluginConfig.ios.pbxproj) {
                  _context3.next = 246;
                  break;
                }

                if (!pluginConfig.ios.pbxproj.addSource) {
                  _context3.next = 126;
                  break;
                }

                _loop = function _loop(source) {
                  // Multiple source files
                  if (source.from) {
                    if (_this.switchToOldDirectoryStructure(pluginSourcePath, source.from)) {
                      log.debug(`Source Copy: Falling back to old directory structure for API(Backward compatibility)`);
                      source.from = 'IOS/IOS/Classes/SwaggersAPIs/*.swift';
                    }
                    var relativeSourcePath = _path3.default.dirname(source.from);
                    var pathToSourceFiles = _path3.default.join(pluginSourcePath, relativeSourcePath);
                    var fileNames = _lodash2.default.filter(_fs2.default.readdirSync(pathToSourceFiles), function (f) {
                      return f.endsWith(_path3.default.extname(source.from));
                    });
                    var _iteratorNormalCompletion8 = true;
                    var _didIteratorError8 = false;
                    var _iteratorError8 = undefined;

                    try {
                      for (var _iterator8 = fileNames[Symbol.iterator](), _step8; !(_iteratorNormalCompletion8 = (_step8 = _iterator8.next()).done); _iteratorNormalCompletion8 = true) {
                        var fileName = _step8.value;

                        var fileNamePath = _path3.default.join(source.path, fileName);
                        containerIosProject.addSourceFile(fileNamePath, null, containerIosProject.findPBXGroupKey({ name: source.group }));
                      }
                    } catch (err) {
                      _didIteratorError8 = true;
                      _iteratorError8 = err;
                    } finally {
                      try {
                        if (!_iteratorNormalCompletion8 && _iterator8.return) {
                          _iterator8.return();
                        }
                      } finally {
                        if (_didIteratorError8) {
                          throw _iteratorError8;
                        }
                      }
                    }
                  } else {
                    // Single source file
                    containerIosProject.addSourceFile(source.path, null, containerIosProject.findPBXGroupKey({ name: source.group }));
                  }
                };

                _iteratorNormalCompletion7 = true;
                _didIteratorError7 = false;
                _iteratorError7 = undefined;
                _context3.prev = 110;

                for (_iterator7 = pluginConfig.ios.pbxproj.addSource[Symbol.iterator](); !(_iteratorNormalCompletion7 = (_step7 = _iterator7.next()).done); _iteratorNormalCompletion7 = true) {
                  source = _step7.value;

                  _loop(source);
                }
                _context3.next = 118;
                break;

              case 114:
                _context3.prev = 114;
                _context3.t3 = _context3['catch'](110);
                _didIteratorError7 = true;
                _iteratorError7 = _context3.t3;

              case 118:
                _context3.prev = 118;
                _context3.prev = 119;

                if (!_iteratorNormalCompletion7 && _iterator7.return) {
                  _iterator7.return();
                }

              case 121:
                _context3.prev = 121;

                if (!_didIteratorError7) {
                  _context3.next = 124;
                  break;
                }

                throw _iteratorError7;

              case 124:
                return _context3.finish(121);

              case 125:
                return _context3.finish(118);

              case 126:
                if (!pluginConfig.ios.pbxproj.addHeader) {
                  _context3.next = 146;
                  break;
                }

                _iteratorNormalCompletion9 = true;
                _didIteratorError9 = false;
                _iteratorError9 = undefined;
                _context3.prev = 130;

                for (_iterator9 = pluginConfig.ios.pbxproj.addHeader[Symbol.iterator](); !(_iteratorNormalCompletion9 = (_step9 = _iterator9.next()).done); _iteratorNormalCompletion9 = true) {
                  header = _step9.value;
                  headerPath = header.path;

                  containerIosProject.addHeaderFile(headerPath, { public: header.public }, containerIosProject.findPBXGroupKey({ name: header.group }));
                }
                _context3.next = 138;
                break;

              case 134:
                _context3.prev = 134;
                _context3.t4 = _context3['catch'](130);
                _didIteratorError9 = true;
                _iteratorError9 = _context3.t4;

              case 138:
                _context3.prev = 138;
                _context3.prev = 139;

                if (!_iteratorNormalCompletion9 && _iterator9.return) {
                  _iterator9.return();
                }

              case 141:
                _context3.prev = 141;

                if (!_didIteratorError9) {
                  _context3.next = 144;
                  break;
                }

                throw _iteratorError9;

              case 144:
                return _context3.finish(141);

              case 145:
                return _context3.finish(138);

              case 146:
                if (!pluginConfig.ios.pbxproj.addFile) {
                  _context3.next = 166;
                  break;
                }

                _iteratorNormalCompletion10 = true;
                _didIteratorError10 = false;
                _iteratorError10 = undefined;
                _context3.prev = 150;

                for (_iterator10 = pluginConfig.ios.pbxproj.addFile[Symbol.iterator](); !(_iteratorNormalCompletion10 = (_step10 = _iterator10.next()).done); _iteratorNormalCompletion10 = true) {
                  _file = _step10.value;

                  containerIosProject.addFile(_file.path, containerIosProject.findPBXGroupKey({ name: _file.group }));
                  // Add target dep in any case for now, will rework later
                  containerIosProject.addTargetDependency(electrodeContainerTarget, [`"${_path3.default.basename(_file.path)}"`]);
                }
                _context3.next = 158;
                break;

              case 154:
                _context3.prev = 154;
                _context3.t5 = _context3['catch'](150);
                _didIteratorError10 = true;
                _iteratorError10 = _context3.t5;

              case 158:
                _context3.prev = 158;
                _context3.prev = 159;

                if (!_iteratorNormalCompletion10 && _iterator10.return) {
                  _iterator10.return();
                }

              case 161:
                _context3.prev = 161;

                if (!_didIteratorError10) {
                  _context3.next = 164;
                  break;
                }

                throw _iteratorError10;

              case 164:
                return _context3.finish(161);

              case 165:
                return _context3.finish(158);

              case 166:
                if (!pluginConfig.ios.pbxproj.addFramework) {
                  _context3.next = 186;
                  break;
                }

                _iteratorNormalCompletion11 = true;
                _didIteratorError11 = false;
                _iteratorError11 = undefined;
                _context3.prev = 170;

                for (_iterator11 = pluginConfig.ios.pbxproj.addFramework[Symbol.iterator](); !(_iteratorNormalCompletion11 = (_step11 = _iterator11.next()).done); _iteratorNormalCompletion11 = true) {
                  framework = _step11.value;

                  containerIosProject.addFramework(framework, { sourceTree: 'BUILT_PRODUCTS_DIR', customFramework: true });
                  containerIosProject.addCopyfileFrameworkCustom(framework);
                }
                _context3.next = 178;
                break;

              case 174:
                _context3.prev = 174;
                _context3.t6 = _context3['catch'](170);
                _didIteratorError11 = true;
                _iteratorError11 = _context3.t6;

              case 178:
                _context3.prev = 178;
                _context3.prev = 179;

                if (!_iteratorNormalCompletion11 && _iterator11.return) {
                  _iterator11.return();
                }

              case 181:
                _context3.prev = 181;

                if (!_didIteratorError11) {
                  _context3.next = 184;
                  break;
                }

                throw _iteratorError11;

              case 184:
                return _context3.finish(181);

              case 185:
                return _context3.finish(178);

              case 186:
                if (!pluginConfig.ios.pbxproj.addProject) {
                  _context3.next = 206;
                  break;
                }

                _iteratorNormalCompletion12 = true;
                _didIteratorError12 = false;
                _iteratorError12 = undefined;
                _context3.prev = 190;

                for (_iterator12 = pluginConfig.ios.pbxproj.addProject[Symbol.iterator](); !(_iteratorNormalCompletion12 = (_step12 = _iterator12.next()).done); _iteratorNormalCompletion12 = true) {
                  project = _step12.value;
                  projectAbsolutePath = `${containerLibrariesPath}/${project.path}/project.pbxproj`;

                  containerIosProject.addProject(projectAbsolutePath, project.path, project.group, electrodeContainerTarget, project.staticLibs);
                }
                _context3.next = 198;
                break;

              case 194:
                _context3.prev = 194;
                _context3.t7 = _context3['catch'](190);
                _didIteratorError12 = true;
                _iteratorError12 = _context3.t7;

              case 198:
                _context3.prev = 198;
                _context3.prev = 199;

                if (!_iteratorNormalCompletion12 && _iterator12.return) {
                  _iterator12.return();
                }

              case 201:
                _context3.prev = 201;

                if (!_didIteratorError12) {
                  _context3.next = 204;
                  break;
                }

                throw _iteratorError12;

              case 204:
                return _context3.finish(201);

              case 205:
                return _context3.finish(198);

              case 206:
                if (!pluginConfig.ios.pbxproj.addStaticLibrary) {
                  _context3.next = 226;
                  break;
                }

                _iteratorNormalCompletion13 = true;
                _didIteratorError13 = false;
                _iteratorError13 = undefined;
                _context3.prev = 210;

                for (_iterator13 = pluginConfig.ios.pbxproj.addStaticLibrary[Symbol.iterator](); !(_iteratorNormalCompletion13 = (_step13 = _iterator13.next()).done); _iteratorNormalCompletion13 = true) {
                  lib = _step13.value;

                  containerIosProject.addStaticLibrary(lib);
                }
                _context3.next = 218;
                break;

              case 214:
                _context3.prev = 214;
                _context3.t8 = _context3['catch'](210);
                _didIteratorError13 = true;
                _iteratorError13 = _context3.t8;

              case 218:
                _context3.prev = 218;
                _context3.prev = 219;

                if (!_iteratorNormalCompletion13 && _iterator13.return) {
                  _iterator13.return();
                }

              case 221:
                _context3.prev = 221;

                if (!_didIteratorError13) {
                  _context3.next = 224;
                  break;
                }

                throw _iteratorError13;

              case 224:
                return _context3.finish(221);

              case 225:
                return _context3.finish(218);

              case 226:
                if (!pluginConfig.ios.pbxproj.addHeaderSearchPath) {
                  _context3.next = 246;
                  break;
                }

                _iteratorNormalCompletion14 = true;
                _didIteratorError14 = false;
                _iteratorError14 = undefined;
                _context3.prev = 230;

                for (_iterator14 = pluginConfig.ios.pbxproj.addHeaderSearchPath[Symbol.iterator](); !(_iteratorNormalCompletion14 = (_step14 = _iterator14.next()).done); _iteratorNormalCompletion14 = true) {
                  _path = _step14.value;

                  containerIosProject.addToHeaderSearchPaths(_path);
                }
                _context3.next = 238;
                break;

              case 234:
                _context3.prev = 234;
                _context3.t9 = _context3['catch'](230);
                _didIteratorError14 = true;
                _iteratorError14 = _context3.t9;

              case 238:
                _context3.prev = 238;
                _context3.prev = 239;

                if (!_iteratorNormalCompletion14 && _iterator14.return) {
                  _iterator14.return();
                }

              case 241:
                _context3.prev = 241;

                if (!_didIteratorError14) {
                  _context3.next = 244;
                  break;
                }

                throw _iteratorError14;

              case 244:
                return _context3.finish(241);

              case 245:
                return _context3.finish(238);

              case 246:
                _iteratorNormalCompletion4 = true;
                _context3.next = 49;
                break;

              case 249:
                _context3.next = 255;
                break;

              case 251:
                _context3.prev = 251;
                _context3.t10 = _context3['catch'](47);
                _didIteratorError4 = true;
                _iteratorError4 = _context3.t10;

              case 255:
                _context3.prev = 255;
                _context3.prev = 256;

                if (!_iteratorNormalCompletion4 && _iterator4.return) {
                  _iterator4.return();
                }

              case 258:
                _context3.prev = 258;

                if (!_didIteratorError4) {
                  _context3.next = 261;
                  break;
                }

                throw _iteratorError4;

              case 261:
                return _context3.finish(258);

              case 262:
                return _context3.finish(255);

              case 263:
                _context3.next = 265;
                return this.addiOSPluginHookClasses(containerIosProject, plugins, paths);

              case 265:
                _fs2.default.writeFileSync(containerProjectPath, containerIosProject.writeSync());

                log.debug(`[=== Completed container hull filling ===]`);

              case 267:
              case 'end':
                return _context3.stop();
            }
          }
        }, _callee3, this, [[14, 26, 30, 38], [31,, 33, 37], [47, 251, 255, 263], [67, 71, 75, 83], [76,, 78, 82], [88, 92, 96, 104], [97,, 99, 103], [110, 114, 118, 126], [119,, 121, 125], [130, 134, 138, 146], [139,, 141, 145], [150, 154, 158, 166], [159,, 161, 165], [170, 174, 178, 186], [179,, 181, 185], [190, 194, 198, 206], [199,, 201, 205], [210, 214, 218, 226], [219,, 221, 225], [230, 234, 238, 246], [239,, 241, 245], [256,, 258, 262]]);
      }));

      function fillContainerHull(_x9, _x10, _x11, _x12) {
        return _ref4.apply(this, arguments);
      }

      return fillContainerHull;
    }()

    // Code to keep backward compatibility

  }, {
    key: 'switchToOldDirectoryStructure',
    value: function switchToOldDirectoryStructure(pluginSourcePath, tail) {
      // This is to check if the api referenced during container generation is created using the old or new directory structure to help keep the backward compatibility.
      if (_path3.default.dirname(tail) === `IOS` && _fs2.default.existsSync(_path3.default.join(pluginSourcePath, _path3.default.dirname('IOS/IOS/Classes/SwaggersAPIs')))) {
        return true;
      }
      return false;
    }
  }, {
    key: 'buildiOSPluginsViews',
    value: function () {
      var _ref5 = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee4(plugins, mustacheView) {
        var pluginsView, _iteratorNormalCompletion15, _didIteratorError15, _iteratorError15, _iterator15, _step15, plugin, pluginConfig, iosPluginHook, containerHeader;

        return regeneratorRuntime.wrap(function _callee4$(_context4) {
          while (1) {
            switch (_context4.prev = _context4.next) {
              case 0:
                _context4.prev = 0;
                pluginsView = [];

                log.debug(`===iOS: building iOS plugin views`);
                _iteratorNormalCompletion15 = true;
                _didIteratorError15 = false;
                _iteratorError15 = undefined;
                _context4.prev = 6;
                _iterator15 = plugins[Symbol.iterator]();

              case 8:
                if (_iteratorNormalCompletion15 = (_step15 = _iterator15.next()).done) {
                  _context4.next = 21;
                  break;
                }

                plugin = _step15.value;

                if (!(plugin.name === 'react-native')) {
                  _context4.next = 12;
                  break;
                }

                return _context4.abrupt('continue', 18);

              case 12:
                _context4.next = 14;
                return _ernCore.manifest.getPluginConfig(plugin);

              case 14:
                pluginConfig = _context4.sent;
                iosPluginHook = pluginConfig.ios.pluginHook;
                containerHeader = pluginConfig.ios.containerPublicHeader;

                if (iosPluginHook.configurable) {
                  pluginsView.push({
                    'name': iosPluginHook.name,
                    'lcname': iosPluginHook.name.charAt(0).toLowerCase() + iosPluginHook.name.slice(1),
                    'configurable': iosPluginHook.configurable,
                    'containerHeader': containerHeader
                  });
                } else {
                  pluginsView.push({
                    'configurable': iosPluginHook.configurable,
                    'containerHeader': containerHeader
                  });
                }

              case 18:
                _iteratorNormalCompletion15 = true;
                _context4.next = 8;
                break;

              case 21:
                _context4.next = 27;
                break;

              case 23:
                _context4.prev = 23;
                _context4.t0 = _context4['catch'](6);
                _didIteratorError15 = true;
                _iteratorError15 = _context4.t0;

              case 27:
                _context4.prev = 27;
                _context4.prev = 28;

                if (!_iteratorNormalCompletion15 && _iterator15.return) {
                  _iterator15.return();
                }

              case 30:
                _context4.prev = 30;

                if (!_didIteratorError15) {
                  _context4.next = 33;
                  break;
                }

                throw _iteratorError15;

              case 33:
                return _context4.finish(30);

              case 34:
                return _context4.finish(27);

              case 35:

                mustacheView.plugins = pluginsView;
                _context4.next = 42;
                break;

              case 38:
                _context4.prev = 38;
                _context4.t1 = _context4['catch'](0);

                log.error('[buildiOSPluginsViews] Something went wrong: ' + _context4.t1);
                throw _context4.t1;

              case 42:
              case 'end':
                return _context4.stop();
            }
          }
        }, _callee4, this, [[0, 38], [6, 23, 27, 35], [28,, 30, 34]]);
      }));

      function buildiOSPluginsViews(_x13, _x14) {
        return _ref5.apply(this, arguments);
      }

      return buildiOSPluginsViews;
    }()
  }, {
    key: 'addiOSPluginHookClasses',
    value: function () {
      var _ref6 = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee5(containerIosProject, plugins, paths) {
        var _iteratorNormalCompletion16, _didIteratorError16, _iteratorError16, _iterator16, _step16, plugin, pluginConfig, iOSPluginHook;

        return regeneratorRuntime.wrap(function _callee5$(_context5) {
          while (1) {
            switch (_context5.prev = _context5.next) {
              case 0:
                _context5.prev = 0;

                log.debug(`[=== iOS: Adding plugin hook classes ===]`);

                _iteratorNormalCompletion16 = true;
                _didIteratorError16 = false;
                _iteratorError16 = undefined;
                _context5.prev = 5;
                _iterator16 = plugins[Symbol.iterator]();

              case 7:
                if (_iteratorNormalCompletion16 = (_step16 = _iterator16.next()).done) {
                  _context5.next = 36;
                  break;
                }

                plugin = _step16.value;

                if (!(plugin.name === 'react-native')) {
                  _context5.next = 11;
                  break;
                }

                return _context5.abrupt('continue', 33);

              case 11:
                _context5.next = 13;
                return _ernCore.manifest.getPluginConfig(plugin);

              case 13:
                pluginConfig = _context5.sent;

                if (pluginConfig.ios) {
                  _context5.next = 17;
                  break;
                }

                log.warn(`${plugin.name} does not have any injection configuration for iOS`);
                return _context5.abrupt('continue', 33);

              case 17:
                iOSPluginHook = pluginConfig.ios.pluginHook;

                if (!iOSPluginHook) {
                  _context5.next = 33;
                  break;
                }

                if (!iOSPluginHook.header) {
                  _context5.next = 27;
                  break;
                }

                log.debug(`Adding ${iOSPluginHook.name}.h`);

                if (pluginConfig.path) {
                  _context5.next = 23;
                  break;
                }

                throw new Error(`No plugin config path was set. Cannot proceed.`);

              case 23:
                _shelljs2.default.cp(`${pluginConfig.path}/${iOSPluginHook.name}.h`, `${paths.outFolder}/ios/ElectrodeContainer/`);
                (0, _utils.throwIfShellCommandFailed)();
                containerIosProject.addHeaderFile(`${iOSPluginHook.name}.h`, { public: true }, containerIosProject.findPBXGroupKey({ name: 'ElectrodeContainer' }));
                containerIosProject.addSourceFile(`${iOSPluginHook.name}.m`, null, containerIosProject.findPBXGroupKey({ name: 'ElectrodeContainer' }));

              case 27:
                if (!iOSPluginHook.source) {
                  _context5.next = 33;
                  break;
                }

                log.debug(`Adding ${iOSPluginHook.name}.m`);

                if (pluginConfig.path) {
                  _context5.next = 31;
                  break;
                }

                throw new Error(`No plugin config path was set. Cannot proceed.`);

              case 31:
                _shelljs2.default.cp(`${pluginConfig.path}/${iOSPluginHook.name}.m`, `${paths.outFolder}/ios/ElectrodeContainer/`);
                (0, _utils.throwIfShellCommandFailed)();

              case 33:
                _iteratorNormalCompletion16 = true;
                _context5.next = 7;
                break;

              case 36:
                _context5.next = 42;
                break;

              case 38:
                _context5.prev = 38;
                _context5.t0 = _context5['catch'](5);
                _didIteratorError16 = true;
                _iteratorError16 = _context5.t0;

              case 42:
                _context5.prev = 42;
                _context5.prev = 43;

                if (!_iteratorNormalCompletion16 && _iterator16.return) {
                  _iterator16.return();
                }

              case 45:
                _context5.prev = 45;

                if (!_didIteratorError16) {
                  _context5.next = 48;
                  break;
                }

                throw _iteratorError16;

              case 48:
                return _context5.finish(45);

              case 49:
                return _context5.finish(42);

              case 50:

                log.debug(`[=== iOS: Done adding plugin hook classes ===]`);
                _context5.next = 57;
                break;

              case 53:
                _context5.prev = 53;
                _context5.t1 = _context5['catch'](0);

                log.error('[addiOSPluginHookClasses] Something went wrong: ' + _context5.t1);
                throw _context5.t1;

              case 57:
              case 'end':
                return _context5.stop();
            }
          }
        }, _callee5, this, [[0, 53], [5, 38, 42, 50], [43,, 45, 49]]);
      }));

      function addiOSPluginHookClasses(_x15, _x16, _x17) {
        return _ref6.apply(this, arguments);
      }

      return addiOSPluginHookClasses;
    }()
  }, {
    key: 'getIosContainerProject',
    value: function () {
      var _ref7 = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee6(containerProjectPath) {
        var containerProject;
        return regeneratorRuntime.wrap(function _callee6$(_context6) {
          while (1) {
            switch (_context6.prev = _context6.next) {
              case 0:
                containerProject = _xcodeErn2.default.project(containerProjectPath);
                return _context6.abrupt('return', new Promise(function (resolve, reject) {
                  containerProject.parse(function (err) {
                    if (err) {
                      reject(err);
                    }
                    resolve(containerProject);
                  });
                }));

              case 2:
              case 'end':
                return _context6.stop();
            }
          }
        }, _callee6, this);
      }));

      function getIosContainerProject(_x18) {
        return _ref7.apply(this, arguments);
      }

      return getIosContainerProject;
    }()
  }, {
    key: 'name',
    get: function get() {
      return 'IosGenerator';
    }
  }, {
    key: 'platform',
    get: function get() {
      return 'ios';
    }
  }]);

  return IosGenerator;
}();

exports.default = IosGenerator;
//# sourceMappingURL=IosGenerator.js.map