'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.downloadPluginSource = exports.runYarnUsingMiniAppDeltas = exports.generateMiniAppsComposite = exports.reactNativeBundleIos = exports.reactNativeBundleAndroid = exports.bundleMiniApps = undefined;

var bundleMiniApps = exports.bundleMiniApps = function () {
  var _ref = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee(miniapps, paths, platform) {
    var _ref2 = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : {},
        pathToYarnLock = _ref2.pathToYarnLock;

    var miniAppsPaths, _iteratorNormalCompletion, _didIteratorError, _iteratorError, _iterator, _step, miniapp;

    return regeneratorRuntime.wrap(function _callee$(_context) {
      while (1) {
        switch (_context.prev = _context.next) {
          case 0:
            _context.prev = 0;

            log.debug(`[=== Starting mini apps bundling ===]`);

            // Specific case where we use container gen to generate
            // container for runner and we want to bundle the local miniapp

            if (!(miniapps.length === 1 && miniapps[0].localPath)) {
              _context.next = 7;
              break;
            }

            _shelljs2.default.cd(miniapps[0].localPath);
            throwIfShellCommandFailed();
            _context.next = 29;
            break;

          case 7:
            miniAppsPaths = [];
            _iteratorNormalCompletion = true;
            _didIteratorError = false;
            _iteratorError = undefined;
            _context.prev = 11;

            for (_iterator = miniapps[Symbol.iterator](); !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
              miniapp = _step.value;

              if (miniapp.localPath) {
                log.debug(`[bundleMiniApps] Using local path ${miniapp.localPath} for ${miniapp.name}`);
                miniAppsPaths.push(DependencyPath.fromFileSystemPath(miniapp.localPath));
              } else if (miniapp.packagePath) {
                log.debug(`[bundleMiniApps] Using package path ${miniapp.packagePath} for ${miniapp.name}`);
                miniAppsPaths.push(miniapp.packagePath);
              } else {
                log.debug(`[bundleMiniApps] Using built path for ${miniapp.name}`);
                miniAppsPaths.push(new DependencyPath(new Dependency(miniapp.name, {
                  scope: miniapp.scope,
                  version: miniapp.version
                }).toString()));
              }
            }
            _context.next = 19;
            break;

          case 15:
            _context.prev = 15;
            _context.t0 = _context['catch'](11);
            _didIteratorError = true;
            _iteratorError = _context.t0;

          case 19:
            _context.prev = 19;
            _context.prev = 20;

            if (!_iteratorNormalCompletion && _iterator.return) {
              _iterator.return();
            }

          case 22:
            _context.prev = 22;

            if (!_didIteratorError) {
              _context.next = 25;
              break;
            }

            throw _iteratorError;

          case 25:
            return _context.finish(22);

          case 26:
            return _context.finish(19);

          case 27:
            _context.next = 29;
            return generateMiniAppsComposite(miniAppsPaths, paths.compositeMiniApp, { pathToYarnLock });

          case 29:

            // Clear react packager cache beforehand to avoid surprises ...
            clearReactPackagerCache();

            if (!(platform === 'android')) {
              _context.next = 36;
              break;
            }

            log.debug(`Bundling miniapp(s) for Android`);
            _context.next = 34;
            return reactNativeBundleAndroid(paths);

          case 34:
            _context.next = 40;
            break;

          case 36:
            if (!(platform === 'ios')) {
              _context.next = 40;
              break;
            }

            log.debug(`Bundling miniapp(s) for iOS`);
            _context.next = 40;
            return reactNativeBundleIos(paths);

          case 40:

            log.debug(`[=== Completed mini apps bundling ===]`);
            _context.next = 47;
            break;

          case 43:
            _context.prev = 43;
            _context.t1 = _context['catch'](0);

            log.error('[bundleMiniApps] Something went wrong: ' + _context.t1);
            throw _context.t1;

          case 47:
          case 'end':
            return _context.stop();
        }
      }
    }, _callee, this, [[0, 43], [11, 15, 19, 27], [20,, 22, 26]]);
  }));

  return function bundleMiniApps(_x, _x2, _x3) {
    return _ref.apply(this, arguments);
  };
}();

var reactNativeBundleAndroid = exports.reactNativeBundleAndroid = function () {
  var _ref3 = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee2(paths) {
    return regeneratorRuntime.wrap(function _callee2$(_context2) {
      while (1) {
        switch (_context2.prev = _context2.next) {
          case 0:
            return _context2.abrupt('return', _ernCore.reactnative.bundle({
              entryFile: 'index.android.js',
              dev: false,
              bundleOutput: `${paths.outFolder}/android/lib/src/main/assets/index.android.bundle`,
              platform: 'android',
              assetsDest: `${paths.outFolder}/android/lib/src/main/res`
            }));

          case 1:
          case 'end':
            return _context2.stop();
        }
      }
    }, _callee2, this);
  }));

  return function reactNativeBundleAndroid(_x5) {
    return _ref3.apply(this, arguments);
  };
}();

var reactNativeBundleIos = exports.reactNativeBundleIos = function () {
  var _ref4 = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee3(paths) {
    var miniAppOutFolder;
    return regeneratorRuntime.wrap(function _callee3$(_context3) {
      while (1) {
        switch (_context3.prev = _context3.next) {
          case 0:
            miniAppOutFolder = `${paths.outFolder}/ios/ElectrodeContainer/Libraries/MiniApp`;


            if (!_fs2.default.existsSync(miniAppOutFolder)) {
              _shelljs2.default.mkdir('-p', miniAppOutFolder);
              throwIfShellCommandFailed();
            }

            return _context3.abrupt('return', _ernCore.reactnative.bundle({
              entryFile: 'index.ios.js',
              dev: false,
              bundleOutput: `${miniAppOutFolder}/MiniApp.jsbundle`,
              platform: 'ios',
              assetsDest: `${miniAppOutFolder}`
            }));

          case 3:
          case 'end':
            return _context3.stop();
        }
      }
    }, _callee3, this);
  }));

  return function reactNativeBundleIos(_x6) {
    return _ref4.apply(this, arguments);
  };
}();

var generateMiniAppsComposite = exports.generateMiniAppsComposite = function () {
  var _ref5 = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee4(miniappsPaths, outDir) {
    var _ref6 = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {},
        pathToYarnLock = _ref6.pathToYarnLock;

    var compositePackageJson, miniAppsPackages, yarnLock, miniAppsDeltas, _iteratorNormalCompletion2, _didIteratorError2, _iteratorError2, _iterator2, _step2, miniappPath, packageJson, entryIndexJsContent, _iteratorNormalCompletion3, _didIteratorError3, _iteratorError3, _iterator3, _step3, dependency, compositeBabelRc, moduleResolverAliases, _iteratorNormalCompletion4, _didIteratorError4, _iteratorError4, _iterator4, _step4, _dependency, miniAppPackageJsonPath, miniAppPackageJson, miniAppName, _iteratorNormalCompletion5, _didIteratorError5, _iteratorError5, _iterator5, _step5, babelPlugin, _iteratorNormalCompletion6, _didIteratorError6, _iteratorError6, _iterator6, _step6, x, item, _iteratorNormalCompletion7, _didIteratorError7, _iteratorError7, _iterator7, _step7, aliasKey, pathToCodePushNodeModuleDir, pathToReactNativeNodeModuleDir, reactNativePackageJson;

    return regeneratorRuntime.wrap(function _callee4$(_context4) {
      while (1) {
        switch (_context4.prev = _context4.next) {
          case 0:
            _shelljs2.default.mkdir('-p', outDir);
            _shelljs2.default.cd(outDir);
            throwIfShellCommandFailed();

            compositePackageJson = {};

            if (!pathToYarnLock) {
              _context4.next = 27;
              break;
            }

            if (!_lodash2.default.some(miniappsPaths, function (p) {
              return p.isAFileSystemPath || p.isAGitPath;
            })) {
              _context4.next = 7;
              break;
            }

            throw new Error('[generateMiniAppsComposite] When providing a yarn lock you cannot pass MiniApps paths with file or git scheme');

          case 7:
            miniAppsPackages = _lodash2.default.map(miniappsPaths, function (p) {
              return Dependency.fromPath(p);
            });

            if (!_lodash2.default.some(miniAppsPackages, function (m) {
              return !m.isVersioned;
            })) {
              _context4.next = 10;
              break;
            }

            throw new Error('[generateMiniAppsComposite] When providing a yarn lock you cannot pass MiniApps without an explicit version');

          case 10:
            if (_fs2.default.existsSync(pathToYarnLock)) {
              _context4.next = 12;
              break;
            }

            throw new Error(`[generateMiniAppsComposite] Path to yarn.lock does not exists (${pathToYarnLock})`);

          case 12:

            log.debug(`Copying yarn.lock to ${outDir}`);
            _shelljs2.default.cp(pathToYarnLock, outDir);
            throwIfShellCommandFailed();

            yarnLock = _fs2.default.readFileSync(pathToYarnLock, 'utf8');
            miniAppsDeltas = getMiniAppsDeltas(miniAppsPackages, yarnLock);


            log.debug(`[generateMiniAppsComposite] miniAppsDeltas: ${JSON.stringify(miniAppsDeltas)}`);

            // Create initial package.json
            compositePackageJson.dependencies = getPackageJsonDependenciesUsingMiniAppDeltas(miniAppsDeltas, yarnLock);
            // This helps to start the local packager withing the comopsiteMiniApps folder for debugging multiple miniapps
            compositePackageJson.scripts = { 'start': 'node node_modules/react-native/local-cli/cli.js start' };
            _fs2.default.writeFileSync(_path2.default.join(outDir, 'package.json'), JSON.stringify(compositePackageJson, null, 2), 'utf8');

            // Now that the composite package.json is similar to the one used to generated yarn.lock
            // we can run yarn install to get back to the exact same dependency graph as the previously
            // generated composite
            _context4.next = 23;
            return _ernCore.yarn.install();

          case 23:
            _context4.next = 25;
            return runYarnUsingMiniAppDeltas(miniAppsDeltas);

          case 25:
            _context4.next = 57;
            break;

          case 27:
            // No yarn.lock path was provided, just add miniapps one by one
            log.debug(`[generateMiniAppsComposite] no yarn lock provided`);
            _iteratorNormalCompletion2 = true;
            _didIteratorError2 = false;
            _iteratorError2 = undefined;
            _context4.prev = 31;
            _iterator2 = miniappsPaths[Symbol.iterator]();

          case 33:
            if (_iteratorNormalCompletion2 = (_step2 = _iterator2.next()).done) {
              _context4.next = 40;
              break;
            }

            miniappPath = _step2.value;
            _context4.next = 37;
            return _ernCore.yarn.add(miniappPath);

          case 37:
            _iteratorNormalCompletion2 = true;
            _context4.next = 33;
            break;

          case 40:
            _context4.next = 46;
            break;

          case 42:
            _context4.prev = 42;
            _context4.t0 = _context4['catch'](31);
            _didIteratorError2 = true;
            _iteratorError2 = _context4.t0;

          case 46:
            _context4.prev = 46;
            _context4.prev = 47;

            if (!_iteratorNormalCompletion2 && _iterator2.return) {
              _iterator2.return();
            }

          case 49:
            _context4.prev = 49;

            if (!_didIteratorError2) {
              _context4.next = 52;
              break;
            }

            throw _iteratorError2;

          case 52:
            return _context4.finish(49);

          case 53:
            return _context4.finish(46);

          case 54:
            packageJson = JSON.parse(_fs2.default.readFileSync(`${outDir}/package.json`, 'utf-8'));

            packageJson.scripts = { 'start': 'node node_modules/react-native/local-cli/cli.js start' };
            _fs2.default.writeFileSync(_path2.default.join(outDir, 'package.json'), JSON.stringify(packageJson, null, 2), 'utf8');

          case 57:
            entryIndexJsContent = '';


            compositePackageJson = JSON.parse(_fs2.default.readFileSync('package.json', 'utf8'));
            _iteratorNormalCompletion3 = true;
            _didIteratorError3 = false;
            _iteratorError3 = undefined;
            _context4.prev = 62;
            for (_iterator3 = Object.keys(compositePackageJson.dependencies)[Symbol.iterator](); !(_iteratorNormalCompletion3 = (_step3 = _iterator3.next()).done); _iteratorNormalCompletion3 = true) {
              dependency = _step3.value;

              entryIndexJsContent += `import '${dependency}'\n`;
            }

            _context4.next = 70;
            break;

          case 66:
            _context4.prev = 66;
            _context4.t1 = _context4['catch'](62);
            _didIteratorError3 = true;
            _iteratorError3 = _context4.t1;

          case 70:
            _context4.prev = 70;
            _context4.prev = 71;

            if (!_iteratorNormalCompletion3 && _iterator3.return) {
              _iterator3.return();
            }

          case 73:
            _context4.prev = 73;

            if (!_didIteratorError3) {
              _context4.next = 76;
              break;
            }

            throw _iteratorError3;

          case 76:
            return _context4.finish(73);

          case 77:
            return _context4.finish(70);

          case 78:
            log.debug(`Removing .babelrc files from all modules`);
            _shelljs2.default.rm('-rf', 'node_modules/**/.babelrc');
            throwIfShellCommandFailed();

            log.debug(`Creating top level composite .babelrc`);
            compositeBabelRc = { 'presets': ['react-native'], 'plugins': []

              // Ugly hacky way of handling module-resolver babel plugin
              // At least it has some guarantees to make it safer but its just a temporary
              // solution until we figure out a more proper way of handling this plugin
            };
            log.debug(`Taking care of potential Babel plugins used by MiniApps`);
            moduleResolverAliases = {};
            _iteratorNormalCompletion4 = true;
            _didIteratorError4 = false;
            _iteratorError4 = undefined;
            _context4.prev = 88;
            _iterator4 = Object.keys(compositePackageJson.dependencies)[Symbol.iterator]();

          case 90:
            if (_iteratorNormalCompletion4 = (_step4 = _iterator4.next()).done) {
              _context4.next = 209;
              break;
            }

            _dependency = _step4.value;
            miniAppPackageJsonPath = `${outDir}/node_modules/${_dependency}/package.json`;
            miniAppPackageJson = void 0;
            _context4.prev = 94;

            miniAppPackageJson = JSON.parse(_fs2.default.readFileSync(miniAppPackageJsonPath, 'utf-8'));
            _context4.next = 101;
            break;

          case 98:
            _context4.prev = 98;
            _context4.t2 = _context4['catch'](94);
            return _context4.abrupt('continue', 206);

          case 101:
            miniAppName = miniAppPackageJson.name;

            if (!miniAppPackageJson.babel) {
              _context4.next = 206;
              break;
            }

            if (!miniAppPackageJson.babel.plugins) {
              _context4.next = 203;
              break;
            }

            _iteratorNormalCompletion5 = true;
            _didIteratorError5 = false;
            _iteratorError5 = undefined;
            _context4.prev = 107;
            _iterator5 = miniAppPackageJson.babel.plugins[Symbol.iterator]();

          case 109:
            if (_iteratorNormalCompletion5 = (_step5 = _iterator5.next()).done) {
              _context4.next = 189;
              break;
            }

            babelPlugin = _step5.value;

            if (!Array.isArray(babelPlugin)) {
              _context4.next = 185;
              break;
            }

            if (!babelPlugin.includes('module-resolver')) {
              _context4.next = 182;
              break;
            }

            // Copy over module-resolver plugin & config to top level composite .babelrc
            log.debug(`Taking care of module-resolver Babel plugin for ${miniAppName} MiniApp`);

            if (!(compositeBabelRc.plugins.length === 0)) {
              _context4.next = 145;
              break;
            }

            // First MiniApp to add module-resolver plugin & config
            // easy enough, we just copy over the plugin & config
            compositeBabelRc.plugins.push(babelPlugin);
            _iteratorNormalCompletion6 = true;
            _didIteratorError6 = false;
            _iteratorError6 = undefined;
            _context4.prev = 119;
            _iterator6 = babelPlugin[Symbol.iterator]();

          case 121:
            if (_iteratorNormalCompletion6 = (_step6 = _iterator6.next()).done) {
              _context4.next = 129;
              break;
            }

            x = _step6.value;

            if (!(x instanceof Object && x.alias)) {
              _context4.next = 126;
              break;
            }

            moduleResolverAliases = x.alias;
            return _context4.abrupt('break', 129);

          case 126:
            _iteratorNormalCompletion6 = true;
            _context4.next = 121;
            break;

          case 129:
            _context4.next = 135;
            break;

          case 131:
            _context4.prev = 131;
            _context4.t3 = _context4['catch'](119);
            _didIteratorError6 = true;
            _iteratorError6 = _context4.t3;

          case 135:
            _context4.prev = 135;
            _context4.prev = 136;

            if (!_iteratorNormalCompletion6 && _iterator6.return) {
              _iterator6.return();
            }

          case 138:
            _context4.prev = 138;

            if (!_didIteratorError6) {
              _context4.next = 141;
              break;
            }

            throw _iteratorError6;

          case 141:
            return _context4.finish(138);

          case 142:
            return _context4.finish(135);

          case 143:
            _context4.next = 180;
            break;

          case 145:
            _context4.t4 = regeneratorRuntime.keys(babelPlugin);

          case 146:
            if ((_context4.t5 = _context4.t4()).done) {
              _context4.next = 180;
              break;
            }

            item = _context4.t5.value;

            if (!(item instanceof Object && item.alias)) {
              _context4.next = 178;
              break;
            }

            _iteratorNormalCompletion7 = true;
            _didIteratorError7 = false;
            _iteratorError7 = undefined;
            _context4.prev = 152;
            _iterator7 = Object.keys(item.alias)[Symbol.iterator]();

          case 154:
            if (_iteratorNormalCompletion7 = (_step7 = _iterator7.next()).done) {
              _context4.next = 164;
              break;
            }

            aliasKey = _step7.value;

            if (!(moduleResolverAliases[aliasKey] && moduleResolverAliases[aliasKey] !== item.alias[aliasKey])) {
              _context4.next = 160;
              break;
            }

            throw new Error(`Babel module-resolver alias conflict`);

          case 160:
            if (!moduleResolverAliases[aliasKey]) {
              moduleResolverAliases[aliasKey] = item.alias[aliasKey];
            }

          case 161:
            _iteratorNormalCompletion7 = true;
            _context4.next = 154;
            break;

          case 164:
            _context4.next = 170;
            break;

          case 166:
            _context4.prev = 166;
            _context4.t6 = _context4['catch'](152);
            _didIteratorError7 = true;
            _iteratorError7 = _context4.t6;

          case 170:
            _context4.prev = 170;
            _context4.prev = 171;

            if (!_iteratorNormalCompletion7 && _iterator7.return) {
              _iterator7.return();
            }

          case 173:
            _context4.prev = 173;

            if (!_didIteratorError7) {
              _context4.next = 176;
              break;
            }

            throw _iteratorError7;

          case 176:
            return _context4.finish(173);

          case 177:
            return _context4.finish(170);

          case 178:
            _context4.next = 146;
            break;

          case 180:
            _context4.next = 183;
            break;

          case 182:
            log.warn(`Unsupported Babel plugin type ${babelPlugin.toString()} in ${miniAppName} MiniApp`);

          case 183:
            _context4.next = 186;
            break;

          case 185:
            log.warn(`Unsupported Babel plugin type ${babelPlugin.toString()} in ${miniAppName} MiniApp`);

          case 186:
            _iteratorNormalCompletion5 = true;
            _context4.next = 109;
            break;

          case 189:
            _context4.next = 195;
            break;

          case 191:
            _context4.prev = 191;
            _context4.t7 = _context4['catch'](107);
            _didIteratorError5 = true;
            _iteratorError5 = _context4.t7;

          case 195:
            _context4.prev = 195;
            _context4.prev = 196;

            if (!_iteratorNormalCompletion5 && _iterator5.return) {
              _iterator5.return();
            }

          case 198:
            _context4.prev = 198;

            if (!_didIteratorError5) {
              _context4.next = 201;
              break;
            }

            throw _iteratorError5;

          case 201:
            return _context4.finish(198);

          case 202:
            return _context4.finish(195);

          case 203:
            log.debug(`Removing babel object from ${miniAppName} MiniApp package.json`);
            delete miniAppPackageJson['babel'];
            _fs2.default.writeFileSync(miniAppPackageJsonPath, JSON.stringify(miniAppPackageJson, null, 2), 'utf-8');

          case 206:
            _iteratorNormalCompletion4 = true;
            _context4.next = 90;
            break;

          case 209:
            _context4.next = 215;
            break;

          case 211:
            _context4.prev = 211;
            _context4.t8 = _context4['catch'](88);
            _didIteratorError4 = true;
            _iteratorError4 = _context4.t8;

          case 215:
            _context4.prev = 215;
            _context4.prev = 216;

            if (!_iteratorNormalCompletion4 && _iterator4.return) {
              _iterator4.return();
            }

          case 218:
            _context4.prev = 218;

            if (!_didIteratorError4) {
              _context4.next = 221;
              break;
            }

            throw _iteratorError4;

          case 221:
            return _context4.finish(218);

          case 222:
            return _context4.finish(215);

          case 223:
            _context4.next = 225;
            return writeFile('.babelrc', JSON.stringify(compositeBabelRc, null, 2));

          case 225:
            pathToCodePushNodeModuleDir = `${outDir}/node_modules/react-native-code-push`;
            pathToReactNativeNodeModuleDir = `${outDir}/node_modules/react-native`;
            // If code push plugin is present we need to do some additional work

            if (_fs2.default.existsSync(pathToCodePushNodeModuleDir)) {
              reactNativePackageJson = JSON.parse(_fs2.default.readFileSync(`${pathToReactNativeNodeModuleDir}/package.json`, 'utf8'));

              //
              // The following code will need to be uncommented and properly reworked or included
              // in a different way, once Cart and TYP don't directly depend on code push directly
              // We will work with Cart team in that direction
              //
              // await yarn.add(codePushPlugin.name, codePushPlugin.version)
              // content += `import codePush from "react-native-code-push"\n`
              // content += `codePush.sync()`

              // We need to add some info to package.json for CodePush
              // In order to run, code push needs to find the following in package.json
              // - name & version
              // - react-native in the dependency block
              // TODO :For now we hardcode these values for demo purposes. That being said it
              // might not be needed to do something better because it seems like
              // code push is not making a real use of this data
              // Investigate further.
              // https://github.com/Microsoft/code-push/blob/master/cli/script/command-executor.ts#L1246

              compositePackageJson.dependencies['react-native'] = reactNativePackageJson.version;
              compositePackageJson.name = 'container';
              compositePackageJson.version = '0.0.1';
              _fs2.default.writeFileSync('package.json', JSON.stringify(compositePackageJson, null, 2));
            }

            log.debug(`Creating index.android.js`);
            _context4.next = 231;
            return writeFile('index.android.js', entryIndexJsContent);

          case 231:
            log.debug(`Creating index.ios.js`);
            _context4.next = 234;
            return writeFile('index.ios.js', entryIndexJsContent);

          case 234:
          case 'end':
            return _context4.stop();
        }
      }
    }, _callee4, this, [[31, 42, 46, 54], [47,, 49, 53], [62, 66, 70, 78], [71,, 73, 77], [88, 211, 215, 223], [94, 98], [107, 191, 195, 203], [119, 131, 135, 143], [136,, 138, 142], [152, 166, 170, 178], [171,, 173, 177], [196,, 198, 202], [216,, 218, 222]]);
  }));

  return function generateMiniAppsComposite(_x7, _x8) {
    return _ref5.apply(this, arguments);
  };
}();

var runYarnUsingMiniAppDeltas = exports.runYarnUsingMiniAppDeltas = function () {
  var _ref7 = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee5(miniAppsDeltas) {
    var _iteratorNormalCompletion10, _didIteratorError10, _iteratorError10, _iterator10, _step10, m, miniappPackage, _iteratorNormalCompletion11, _didIteratorError11, _iteratorError11, _iterator11, _step11, _m2, _miniappPackage;

    return regeneratorRuntime.wrap(function _callee5$(_context5) {
      while (1) {
        switch (_context5.prev = _context5.next) {
          case 0:
            if (!miniAppsDeltas.new) {
              _context5.next = 29;
              break;
            }

            _iteratorNormalCompletion10 = true;
            _didIteratorError10 = false;
            _iteratorError10 = undefined;
            _context5.prev = 4;
            _iterator10 = miniAppsDeltas.new[Symbol.iterator]();

          case 6:
            if (_iteratorNormalCompletion10 = (_step10 = _iterator10.next()).done) {
              _context5.next = 15;
              break;
            }

            m = _step10.value;
            miniappPackage = Dependency.fromObject(m);

            log.debug(`Adding new MiniApp ${miniappPackage.toString()}`);
            _context5.next = 12;
            return _ernCore.yarn.add(miniappPackage.path);

          case 12:
            _iteratorNormalCompletion10 = true;
            _context5.next = 6;
            break;

          case 15:
            _context5.next = 21;
            break;

          case 17:
            _context5.prev = 17;
            _context5.t0 = _context5['catch'](4);
            _didIteratorError10 = true;
            _iteratorError10 = _context5.t0;

          case 21:
            _context5.prev = 21;
            _context5.prev = 22;

            if (!_iteratorNormalCompletion10 && _iterator10.return) {
              _iterator10.return();
            }

          case 24:
            _context5.prev = 24;

            if (!_didIteratorError10) {
              _context5.next = 27;
              break;
            }

            throw _iteratorError10;

          case 27:
            return _context5.finish(24);

          case 28:
            return _context5.finish(21);

          case 29:
            if (!miniAppsDeltas.upgraded) {
              _context5.next = 58;
              break;
            }

            _iteratorNormalCompletion11 = true;
            _didIteratorError11 = false;
            _iteratorError11 = undefined;
            _context5.prev = 33;
            _iterator11 = miniAppsDeltas.upgraded[Symbol.iterator]();

          case 35:
            if (_iteratorNormalCompletion11 = (_step11 = _iterator11.next()).done) {
              _context5.next = 44;
              break;
            }

            _m2 = _step11.value;
            _miniappPackage = Dependency.fromObject(_m2);

            log.debug(`Upgrading MiniApp ${_miniappPackage.toString()}`);
            _context5.next = 41;
            return _ernCore.yarn.upgrade(_miniappPackage.path);

          case 41:
            _iteratorNormalCompletion11 = true;
            _context5.next = 35;
            break;

          case 44:
            _context5.next = 50;
            break;

          case 46:
            _context5.prev = 46;
            _context5.t1 = _context5['catch'](33);
            _didIteratorError11 = true;
            _iteratorError11 = _context5.t1;

          case 50:
            _context5.prev = 50;
            _context5.prev = 51;

            if (!_iteratorNormalCompletion11 && _iterator11.return) {
              _iterator11.return();
            }

          case 53:
            _context5.prev = 53;

            if (!_didIteratorError11) {
              _context5.next = 56;
              break;
            }

            throw _iteratorError11;

          case 56:
            return _context5.finish(53);

          case 57:
            return _context5.finish(50);

          case 58:
          case 'end':
            return _context5.stop();
        }
      }
    }, _callee5, this, [[4, 17, 21, 29], [22,, 24, 28], [33, 46, 50, 58], [51,, 53, 57]]);
  }));

  return function runYarnUsingMiniAppDeltas(_x10) {
    return _ref7.apply(this, arguments);
  };
}();

//
// Download the plugin source given a plugin origin
// pluginOrigin: A plugin origin object
// Sample plugin origin objects :
// {
//  "type": "git",
//  "url": "https://github.com/aoriani/ReactNative-StackTracer.git",
//  "version": "0.1.1"
// }
//
// {
//  "type": "npm",
//  "name": "react-native-code-push",
//  "version": "1.16.1-beta"
// }
//
// Note: The plugin will be downloaded locally to the current folder
// For npm origin it will be put in node_modules folder
// For git origin it will be put directly at the root in a folder named after
// the git repo as one would expect
//
// Returns: Absolute path to where the plugin was installed


var downloadPluginSource = exports.downloadPluginSource = function () {
  var _ref8 = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee6(pluginOrigin) {
    var downloadPath, dependency;
    return regeneratorRuntime.wrap(function _callee6$(_context6) {
      while (1) {
        switch (_context6.prev = _context6.next) {
          case 0:
            downloadPath = '';

            if (!(pluginOrigin.type === 'npm')) {
              _context6.next = 8;
              break;
            }

            dependency = new Dependency(pluginOrigin.name, { scope: pluginOrigin.scope, version: pluginOrigin.version });
            _context6.next = 5;
            return _ernCore.yarn.add(DependencyPath.fromString(dependency.toString()));

          case 5:
            if (pluginOrigin.scope) {
              downloadPath = `node_modules/@${pluginOrigin.scope}/${pluginOrigin.name}`;
            } else {
              downloadPath = `node_modules/${pluginOrigin.name}`;
            }
            _context6.next = 16;
            break;

          case 8:
            if (!(pluginOrigin.type === 'git')) {
              _context6.next = 15;
              break;
            }

            if (!pluginOrigin.version) {
              _context6.next = 13;
              break;
            }

            _context6.next = 12;
            return _ernCore.GitUtils.gitClone(pluginOrigin.url, { branch: pluginOrigin.version });

          case 12:
            downloadPath = gitFolderRe.exec(`${pluginOrigin.url}`)[1];

          case 13:
            _context6.next = 16;
            break;

          case 15:
            throw new Error(`Unsupported plugin origin type : ${pluginOrigin.type}`);

          case 16:
            return _context6.abrupt('return', Promise.resolve(`${_shelljs2.default.pwd()}/${downloadPath}`));

          case 17:
          case 'end':
            return _context6.stop();
        }
      }
    }, _callee6, this);
  }));

  return function downloadPluginSource(_x11) {
    return _ref8.apply(this, arguments);
  };
}();

// =============================================================================
// MISC utils
// =============================================================================

// Given a string returns the same string with its first letter capitalized


// =============================================================================
// Async wrappers
// =============================================================================

var writeFile = function () {
  var _ref9 = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee7(filename, data) {
    return regeneratorRuntime.wrap(function _callee7$(_context7) {
      while (1) {
        switch (_context7.prev = _context7.next) {
          case 0:
            return _context7.abrupt('return', new Promise(function (resolve, reject) {
              _fs2.default.writeFile(filename, data, function (err, res) {
                if (err) reject(err);else resolve(res);
              });
            }));

          case 1:
          case 'end':
            return _context7.stop();
        }
      }
    }, _callee7, this);
  }));

  return function writeFile(_x12, _x13) {
    return _ref9.apply(this, arguments);
  };
}();

// =============================================================================
// Shell error helper
// =============================================================================

exports.clearReactPackagerCache = clearReactPackagerCache;
exports.getMiniAppsDeltas = getMiniAppsDeltas;
exports.getPackageJsonDependenciesUsingMiniAppDeltas = getPackageJsonDependenciesUsingMiniAppDeltas;
exports.capitalizeFirstLetter = capitalizeFirstLetter;
exports.throwIfShellCommandFailed = throwIfShellCommandFailed;

var _ernUtil = require('ern-util');

var ernUtil = _interopRequireWildcard(_ernUtil);

var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

var _shelljs = require('shelljs');

var _shelljs2 = _interopRequireDefault(_shelljs);

var _lodash = require('lodash');

var _lodash2 = _interopRequireDefault(_lodash);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _ernCore = require('ern-core');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { return Promise.resolve(value).then(function (value) { step("next", value); }, function (err) { step("throw", err); }); } } return step("next"); }); }; }

var Dependency = ernUtil.Dependency,
    DependencyPath = ernUtil.DependencyPath;


var gitFolderRe = /.*\/(.*).git/;

function clearReactPackagerCache() {
  var TMPDIR = process.env['TMPDIR'];
  if (TMPDIR) {
    _shelljs2.default.rm('-rf', `${TMPDIR}/react-*`);
    throwIfShellCommandFailed();
  }
}

// Using a yarn.lock file content as reference to figure out deltas, group the MiniApps as follow :
// 'new' : The MiniApp is a new one (it was not part of previously generated composite)
// 'same' : The MiniApp is the same (it was part of previously generated composite, with same version)
// 'upgraded' : The MiniApp has a new version (it was part of previously generated composite, but with a different version)
function getMiniAppsDeltas(miniApps, yarnlock) {
  return _lodash2.default.groupBy(miniApps, function (m) {
    var re = new RegExp(`\n${m.withoutVersion().toString()}@(.+):`);
    var match = re.exec(yarnlock);
    if (match === null) {
      return 'new';
    } else {
      return match[1] === m.version ? 'same' : 'upgraded';
    }
  });
}

// Generate package.json dependencies object based on MiniApp deltas
// The  object will contain the same MiniApps at the same versions that were
// used to generate the provided yarn.lock, so that we get back to original state
// conforming with yarn.lock
// It only contains 'same' and 'upgrade' MiniApps (not new ones)
function getPackageJsonDependenciesUsingMiniAppDeltas(miniAppsDeltas, yarnlock) {
  var result = {};

  if (miniAppsDeltas.same) {
    var _iteratorNormalCompletion8 = true;
    var _didIteratorError8 = false;
    var _iteratorError8 = undefined;

    try {
      for (var _iterator8 = miniAppsDeltas.same[Symbol.iterator](), _step8; !(_iteratorNormalCompletion8 = (_step8 = _iterator8.next()).done); _iteratorNormalCompletion8 = true) {
        var m = _step8.value;

        result[`${m.name}`] = m.version;
      }
    } catch (err) {
      _didIteratorError8 = true;
      _iteratorError8 = err;
    } finally {
      try {
        if (!_iteratorNormalCompletion8 && _iterator8.return) {
          _iterator8.return();
        }
      } finally {
        if (_didIteratorError8) {
          throw _iteratorError8;
        }
      }
    }
  }

  if (miniAppsDeltas.upgraded) {
    var _iteratorNormalCompletion9 = true;
    var _didIteratorError9 = false;
    var _iteratorError9 = undefined;

    try {
      for (var _iterator9 = miniAppsDeltas.upgraded[Symbol.iterator](), _step9; !(_iteratorNormalCompletion9 = (_step9 = _iterator9.next()).done); _iteratorNormalCompletion9 = true) {
        var _m = _step9.value;

        var re = new RegExp(`\n${_m.name.toString()}@(.+):`);
        var initialVersion = re.exec(yarnlock)[1];
        result[`${_m.name}`] = initialVersion;
      }
    } catch (err) {
      _didIteratorError9 = true;
      _iteratorError9 = err;
    } finally {
      try {
        if (!_iteratorNormalCompletion9 && _iterator9.return) {
          _iterator9.return();
        }
      } finally {
        if (_didIteratorError9) {
          throw _iteratorError9;
        }
      }
    }
  }

  return result;
}

function capitalizeFirstLetter(str) {
  return `${str.charAt(0).toUpperCase()}${str.slice(1)}`;
}function throwIfShellCommandFailed() {
  var shellError = _shelljs2.default.error();
  if (shellError) {
    throw new Error(shellError);
  }
}
//# sourceMappingURL=utils.js.map